#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 28 21:26:06 2020

@author: oscar
"""

import re as reg
from copy import deepcopy
from sys import exc_info

import pandas as pd

COL_MED = ["date_local", "typename", "caption", "shortcode", "owner_username"]
ERR_SYS = "system error: "


def create_regex(list_terms):
    """

    Parameters
    ----------
    list_terms : TYPE list
        DESCRIPTION. list with the text to apply

    Returns
    -------
    pat : TYPE list
        DESCRIPTION. lislist without @name

    """
    pattern = r"\b@*" + r"\b|\b@*".join(list_terms) + r"\b"
    pat = reg.compile(pattern)
    return pat


class SocialpresIG:
    def __init__(self, df_ig, df_count, brand, auto_n=True):
        """Digital social presence
        This function calculates social presence, comparing it with the
        weight of each digital media

        Parameters
        ----------
        df_ig : TYPE dataframe
            DESCRIPTION. Contains the information collected from social media
            in Instagram
        df_count : TYPE dataframe
            DESCRIPTION. Contains the media information to consult
        media_weights : TYPE dict
            DESCRIPTION.Weights and names the digital media
        brand : TYPE string
            DESCRIPTION. brand analyze
        auto_n : TYPE bolean
            DESCRIPTION. remove mentions of the same account if true

        Returns
        -------
        None.

        """

        self.df_ig = df_ig
        self.df_count = df_count
        self.brand = brand
        self.auto_n = auto_n

    def c_pres_ig(self):
        """Social media presence
        calculates Instagram only number of appearances

        Returns
        -------
        df_med_c
        TYPE dataframe
            DESCRIPTION. each count number of appearances in Instagram

        """

        df_ig = self.df_ig
        df_c = self.df_count
        auto_n = self.auto_n
        df_empty_c = pd.DataFrame(columns=["media", "mentions"])
        method_name = "c_pres_ig"

        try:
            df_ig = df_ig[COL_MED]
            df_ig = deepcopy(df_ig)
            df_ig.date_local = pd.to_datetime(df_ig.date_local)

            df_ig.caption = df_ig.caption.apply(lambda x: str(x))
            df_ig.caption = df_ig.caption.apply(lambda x: x.lower())

            df_m = deepcopy(df_ig)
            for idd, row in df_c.set_index("nombre").iterrows():
                if idd:
                    account_terms_regex = create_regex(
                        [t.strip() for t in row.terminos.split(",")]
                    )
                    df_m[f"{idd}"] = df_m["caption"].str.contains(
                        account_terms_regex, regex=True
                    )

            df_m["auto"] = 0

            if auto_n:
                col_names = df_m.columns[5:-1]
                for col in col_names:
                    for index in range(len(df_m)):
                        temp_1 = df_m[col]
                        if (temp_1[index] == True) & (
                            df_m.owner_username[index] == col
                        ):
                            df_m.auto.iloc[index] = 1
                df_m2 = df_m[df_m.auto == 0]
                df_m2 = df_m2.reset_index(drop=True)
            else:
                df_m2 = df_m

            df_med_c = df_m2.drop(
                columns=["date_local", "typename", "caption", "shortcode", "auto"]
            ).groupby("owner_username")
            df_med_c = df_med_c.sum().T

            df_med_c = pd.DataFrame(df_med_c.T.sum())
            df_med_c = df_med_c.reset_index()
            df_med_c = df_med_c.rename(columns={0: "mentions", "index": "media"})
            df_med_c = df_med_c.sort_values("mentions", ascending=False)
            df_med_c = df_med_c.reset_index(drop=True)

        except KeyError as e_1:
            print(e_1)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_c = df_empty_c

        except AttributeError as e_2:
            print(e_2)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_c = df_empty_c

        except Exception as e_3:
            print(e_3)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_c = df_empty_c

        return df_med_c

    def id_pres_ig(self):
        """Instagram media presence
        calculates Instagram only number of appearances

        Returns
        -------
        df_med_id
        TYPE dataframe
            DESCRIPTION. each count number of appearances in Instagram

        """

        df_ig = self.df_ig
        df_c = self.df_count
        brand = self.brand
        auto_n = self.auto_n
        df_empty_id = pd.DataFrame(
            columns=["date_local", "caption", "owner_username", "shortcode"]
        )
        method_name = "id_pres_ig"

        try:
            df_ig = df_ig[COL_MED]
            df_ig = deepcopy(df_ig)
            df_ig.date_local = pd.to_datetime(df_ig.date_local)

            df_ig.caption = df_ig.caption.apply(lambda x: str(x))
            df_ig.caption = df_ig.caption.apply(lambda x: x.lower())

            df_m = deepcopy(df_ig)
            for idd, row in df_c.set_index("nombre").iterrows():
                if idd:
                    account_terms_regex = create_regex(
                        [t.strip() for t in row.terminos.split(",")]
                    )
                    df_m[f"{idd}"] = df_m["caption"].str.contains(
                        account_terms_regex, regex=True
                    )

            df_m["auto"] = 0

            if auto_n:
                col_names = df_m.columns[5:-1]
                for col in col_names:
                    for index in range(len(df_m)):
                        temp_1 = df_m[col]
                        if (temp_1[index] == True) & (
                            df_m.owner_username[index] == col
                        ):
                            df_m.auto.iloc[index] = 1
                df_m2 = df_m[df_m.auto == 0]
                df_m2 = df_m2.reset_index(drop=True)
            else:
                df_m2 = df_m

            df_med_id = df_m2[
                ["date_local", "caption", "owner_username", "shortcode", brand]
            ]
            df_med_id = df_med_id[df_med_id[brand] == True]
            df_med_id = df_med_id.drop(columns=brand)
            df_med_id = df_med_id.reset_index(drop=True)

        except KeyError as e_1:
            print(e_1)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_id = df_empty_id

        except AttributeError as e_2:
            print(e_2)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_id = df_empty_id

        except Exception as e_3:
            print(e_3)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_id = df_empty_id

        return df_med_id



import sys
from copy import deepcopy

import numpy as np
import pandas as pd

from ..common import general_utils as gu

ERR_SYS = "\nSystem error: "


class Synchronization:
    def __init__(self, df_posts, platform, engagement_rate="engagement_rate_by_post"):
        """
        This functions computes weekday and hour of the posts.

        Parameters
        ----------
        platform:
            type: str
            Determines the name of the column to compute the weekday and hour from.
        engagement_rate:
            type: str
            Type of engagement rate, default='engagement_rate'.
        """

        METHOD_NAME = "__init__"

        if platform == "twitter" or platform == "instagram":
            DATE_COLUMN = "created_at"
        elif platform == "facebook":
            DATE_COLUMN = "created_time"
        else:
            raise RuntimeError("Platform not available.")

        try:
            df_weekday_hour = deepcopy(df_posts[[DATE_COLUMN, engagement_rate]])

            df_weekday_hour["weekday"] = df_weekday_hour[DATE_COLUMN].apply(
                lambda d: d.weekday()
            )
            df_weekday_hour["hour"] = df_weekday_hour[DATE_COLUMN].apply(
                lambda d: d.hour
            )

            df_weekday_hour = df_weekday_hour.drop(columns=[DATE_COLUMN])
            df_weekday_hour["n_posts"] = 1

            self.df_weekday_hour = df_weekday_hour
            self.engagement_rate = engagement_rate

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            self.df_weekday_hour = pd.DataFrame(
                columns=[DATE_COLUMN, engagement_rate, "weekday", "hour", "n_posts"]
            )
            self.engagement_rate = engagement_rate

    def weekday(self, n_groups=10, threshold_group=3):
        """
        This functions computes the engagement rate, activity and sychronization index
        as a function of weekday of the posts.

        Parameters
        ----------
        n_groups:
            type: str
            Number of groups to split the engagement rate and the activity.
            default=10
        threshold_group:
            type: int
            Minimum level of activity to consider in synchronization. default=3.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "weekday"

        engagement_rate = self.engagement_rate
        df_wd = deepcopy(self.df_weekday_hour[["weekday", "n_posts", engagement_rate]])

        try:
            df_wd = (
                df_wd.groupby("weekday")
                .agg({"n_posts": "count", engagement_rate: "mean"})
                .reset_index()
            )

            gu.discretize(
                df_wd, engagement_rate, to_column="eng_group", n_groups=n_groups
            )
            gu.discretize(df_wd, "n_posts", to_column="act_group", n_groups=n_groups)

            eff_grouped = np.zeros(7)
            act_grouped = np.zeros(7)
            for idd, row in df_wd.iterrows():
                eff_grouped[row.weekday] = row.eng_group
                act_grouped[row.weekday] = row.act_group

            df_wd_out = pd.DataFrame(
                {
                    "weekday": range(7),
                    "activity": eff_grouped,
                    "effectivity": act_grouped,
                }
            )

            df_wd_out["synchronization"] = df_wd_out.apply(
                lambda row: (n_groups - 1) ** 2 - (row.effectivity - row.activity) ** 2
                if row.activity >= threshold_group
                else "N/A",
                axis=1,
            )

            return df_wd_out.sort_values("weekday")

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=["weekday", "activity", "effectivity", "synchronization"]
            )

    def hour(self, n_groups=10, threshold_group=3):
        """
        This functions computes the engagement rate, activity and sychronization index
        as a function of hour of the posts.

        Parameters
        ----------
        n_groups:
            type: str
            Number of groups to split the engagement rate and the activity.
            default=10
        threshold_group:
            type: int
            Minimum level of activity to consider in synchronization. default=3.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "hour"

        engagement_rate = self.engagement_rate
        df_h = deepcopy(self.df_weekday_hour[["hour", "n_posts", engagement_rate]])

        try:
            df_h = (
                df_h.groupby("hour")
                .agg({"n_posts": "count", engagement_rate: "mean"})
                .reset_index()
            )

            gu.discretize(
                df_h, engagement_rate, to_column="eng_group", n_groups=n_groups
            )
            gu.discretize(df_h, "n_posts", to_column="act_group", n_groups=n_groups)

            eff_grouped = np.zeros(24)
            act_grouped = np.zeros(24)
            for idd, row in df_h.iterrows():
                eff_grouped[row.hour] = row.eng_group
                act_grouped[row.hour] = row.act_group

            df_h_out = pd.DataFrame(
                {"hour": range(24), "activity": eff_grouped, "effectivity": act_grouped}
            )

            df_h_out["synchronization"] = df_h_out.apply(
                lambda row: (n_groups - 1) ** 2 - (row.effectivity - row.activity) ** 2
                if row.activity >= threshold_group
                else "N/A",
                axis=1,
            )

            return df_h_out.sort_values("hour")

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=["hour", "activity", "effectivity", "synchronization"]
            )

    def weekday_hour(self, n_groups=10, threshold_group=3):
        """
        This functions computes the engagement rate, activity and sychronization index
        as a function of weekday-hour of the posts.

        Parameters
        ----------
        n_groups:
            type: str
            Number of groups to split the engagement rate and the activity.
            default=10
        threshold_group:
            type: int
            Minimum level of activity to consider in synchronization. default=3.

        Returns
        -------
        tuple of DataFrames
        """

        METHOD_NAME = "weekday_hour"

        engagement_rate = self.engagement_rate
        df_wdh = deepcopy(
            self.df_weekday_hour[["weekday", "hour", "n_posts", engagement_rate]]
        )

        try:
            df_wdh = (
                df_wdh.groupby(["weekday", "hour"])
                .agg({"n_posts": "count", engagement_rate: "mean"})
                .reset_index()
            )

            gu.discretize(
                df_wdh, engagement_rate, to_column="eng_group", n_groups=n_groups
            )
            gu.discretize(df_wdh, "n_posts", to_column="act_group", n_groups=n_groups)

            eff_grouped = np.zeros((24, 7))
            act_grouped = np.zeros((24, 7))
            for idd, row in df_wdh.iterrows():
                eff_grouped[row.hour, row.weekday] = row.eng_group
                act_grouped[row.hour, row.weekday] = row.act_group

            eff_df = pd.DataFrame(eff_grouped)
            act_df = pd.DataFrame(act_grouped)
            sync_df = pd.DataFrame(
                np.where(
                    act_grouped >= threshold_group,
                    (n_groups - 1) ** 2 - (eff_grouped - act_grouped) ** 2,
                    "N/A",
                )
            )

            return eff_df, act_df, sync_df

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            eff_df = np.zeros((24, 7))
            act_df = np.zeros((24, 7))
            sync_df = np.zeros((24, 7))
            return eff_df, act_df, sync_df



import sys
from collections import Counter
from copy import deepcopy
from datetime import datetime

import pandas as pd

ERR_SYS = "\nSystem error: "


def join(iterator, seperator):
    """
    This function casts the elements of iterator to strings then merges those strings together with a string
    representation of seperator.
    Had to write a custom join to handle very, very long lists of things. "".join falls appart above 3013.
    params
    iterator: an iterator.  This function makes use of the overload + operator for strings
    seperator: an item of the same class as is contained in our iterator to be added between every pair of instances.
    returns
    The sum of the iterator values with seperator iterposed between each.
    """
    it = map(str, iterator)
    seperator = str(seperator)
    string = next(it, "")
    for s in it:
        string += seperator + s
    return string


unique_join = lambda x: join(x, ",")


class TermTrend:
    def __init__(self, df_tweets):
        """
        This function copies the input DataFrame.

        Parameters
        ----------
        df_tweets:
            type: DataFrame
            Information of the posts.
            This Pandas DataFrame must have columns 'created_at', 'twitter_id',
            'screen_name', 'user_mentions', 'text', 'ac_followers_count'.
        """

        METHOD_NAME = "__init__"

        TWEETS_COLUMNS = [
            "twitter_id",
            "screen_name",
            "created_at",
            "text",
            "ac_followers_count",
            "user_mentions",
            "profile_image",
            "engagement_rate_by_post",
        ]

        try:

            if "engagement_rate_by_post" not in df_tweets.keys():
                df_tweets["engagement_rate_by_post"] = None

            df_tweets_in = deepcopy(df_tweets[TWEETS_COLUMNS])
            if not df_tweets.empty:
                df_tweets_in["created_at"] = pd.to_datetime(
                    df_tweets_in["created_at"]
                )  # revisar con back si se tiene que hacer corrección horaria

            else:
                print("Warning: The dataframe is empty.")

            self.df_tweets_in = df_tweets_in

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            self.df_tweets_in = pd.DataFrame(columns=TWEETS_COLUMNS)

    def time_series(self, level="hour"):
        """
        This function computes the time series for the posts in the input DataFrame.

        Parameters
        ----------
        level:
            type: str
            Determines the level of resolution to plot in the time series.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "time_series"

        df_tweets_out = deepcopy(self.df_tweets_in[["created_at"]])

        try:
            df_tweets_out["mention_counts"] = 1
            df_tweets_out["created_at"] = df_tweets_out["created_at"].apply(
                lambda dt: datetime(
                    year=dt.year, month=dt.month, day=dt.day, hour=dt.hour
                )
            )
            df_tweets_out = (
                df_tweets_out.groupby("created_at")
                .count()
                .reset_index()
                .sort_values("created_at")
            )
            df_tweets_out["level"] = level

            return df_tweets_out

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(columns=["created_at", "mention_counts", "level"])

    def mentions_network(self, n_most_posts=None, min_group=10, max_nodes=None):
        """
        This function computes the mentions network for the posts in the input DataFrame.

        Parameters
        ----------
        n_most_posts:
            type: int
            Determines the number of posts to create the network from.
            Default = None.
        min_group:
            type: int
            Determines the minimum number of mentions that form a group of
            mentions in the visualization. Default = 10.

        Returns
        -------
        Tuple of DataFrames
        """

        METHOD_NAME = "mentions_network"

        try:
            df_tweets_out = deepcopy(
                self.df_tweets_in[
                    [
                        "twitter_id",
                        "screen_name",
                        "user_mentions",
                        "text",
                        "ac_followers_count",
                        "profile_image",
                    ]
                ]
            )
            df_tweets_out = df_tweets_out[~df_tweets_out["user_mentions"].isna()]

            data_nx = df_tweets_out.groupby(["twitter_id", "screen_name"]).agg(
                {
                    "user_mentions": unique_join,
                    "text": "count",
                    "ac_followers_count": "max",
                    "profile_image": "last",
                }
            )

            if not n_most_posts:
                n_most_posts = len(data_nx)

            data_nx = (
                data_nx.sort_values("text", ascending=False)
                .reset_index()
                .head(n_most_posts)
            )

            # Nodes with posts in the input DataFrame
            nodes_ini = data_nx[
                [
                    "twitter_id",
                    "screen_name",
                    "text",
                    "ac_followers_count",
                    "profile_image",
                ]
            ]
            nodes_ini = nodes_ini.rename(
                columns={
                    "twitter_id": "id",
                    "screen_name": "label",
                    "text": "value",
                    "profile_image": "image",
                }
            )
            nodes_ini["title"] = nodes_ini["ac_followers_count"].apply(
                lambda fw: f"Followers: {fw}"
            )
            nodes_ini = nodes_ini.drop(columns=["ac_followers_count"])

            # Nodes without posts in the input DataFrame
            ment_list = []
            for ment in ",".join(data_nx["user_mentions"]).split(","):
                if ment not in list(data_nx["screen_name"]):
                    ment_list.append(ment)
            ment_list = Counter(ment_list)

            id_list = [f"id_{acc}" for acc in ment_list.keys()]

            nodes_fin = pd.DataFrame(
                {
                    "id": id_list,
                    "label": list(ment_list.keys()),
                    "value": list(ment_list.values()),
                }
            )
            nodes_fin["title"] = 'Followers: "unk"'

            # Concatenation of all nodes
            nodes = pd.concat([nodes_ini, nodes_fin]).reset_index()

            nodes_dict = {}
            for _, row in nodes.iterrows():
                nodes_dict[row.label] = row.id

            # Edges
            from_list = []
            to_list = []
            for _, row in data_nx.iterrows():
                for ment in row.user_mentions.split(","):
                    from_list.append(row.twitter_id)
                    to_list.append(nodes_dict[ment])
            edges = pd.DataFrame({"from": from_list, "to": to_list})

            edges["value"] = 1
            edges = edges.groupby(["from", "to"]).count().reset_index()

            # Grouping nodes and edges to
            nodes["id"] = nodes["id"].apply(lambda i: str(i))
            edges["from"] = edges["from"].apply(lambda i: str(i))
            edges["to"] = edges["to"].apply(lambda i: str(i))

            to_number = (
                edges.groupby(["from"])
                .agg({"to": lambda x: ",".join(x), "value": "count"})
                .reset_index()
            )

            one_to_edges = to_number[to_number["value"].eq(1)]["from"]
            one_to_edges = list(
                set(one_to_edges) - set(edges[edges["to"].isin(one_to_edges)]["to"])
            )

            group_edges_from = (
                edges[edges["from"].isin(one_to_edges)]
                .groupby("to")
                .agg({"value": "count", "from": lambda x: ",".join(x)})
                .reset_index()
            )

            new_nodes_from = group_edges_from[group_edges_from["value"] >= min_group]

            if not new_nodes_from.empty:
                new_nodes_from["label"] = new_nodes_from.apply(
                    lambda row: f'{row["value"]} usuarios', axis=1
                )
                new_nodes_from["title"] = new_nodes_from.apply(
                    lambda row: f'Se agruparon: {row["value"]} usuarios', axis=1
                )
                new_nodes_from["image"] = "group_from"
                new_nodes_from["id"] = new_nodes_from.apply(
                    lambda row: f'{row["value"]}_a_{row["to"]}', axis=1
                )

                for _, row in new_nodes_from.iterrows():
                    edge_remove = row["from"].split(",")
                    nodes = nodes[~nodes["id"].isin(edge_remove)]
                    edges = edges[~edges["from"].isin(edge_remove)]

                new_nodes_from = new_nodes_from.drop(columns=["from", "to"])
                nodes = pd.concat([nodes, new_nodes_from])
                try:
                    nodes = nodes.drop(columns=["index"])
                except Exception:
                    pass

                new_edges = nodes[nodes["image"].eq("group_from")]
                new_edges["to"] = new_edges["id"].apply(
                    lambda index: index.split("_", 2)[2]
                )
                new_edges = new_edges.rename(columns={"id": "from"})
                new_edges = new_edges[["from", "to", "value"]]
                edges = pd.concat([edges, new_edges])

            # Grouping nodes and edges from

            nodes["id"] = nodes["id"].apply(lambda i: str(i))
            edges["from"] = edges["from"].apply(lambda i: str(i))
            edges["to"] = edges["to"].apply(lambda i: str(i))

            from_number = (
                edges.groupby(["to"])
                .agg({"from": lambda x: ",".join(x), "value": "count"})
                .reset_index()
            )

            one_from_edges = from_number[from_number["value"].eq(1)]["to"]
            one_from_edges = list(
                set(one_from_edges)
                - set(edges[edges["from"].isin(one_from_edges)]["from"])
            )

            group_edges_to = (
                edges[edges["to"].isin(one_from_edges)]
                .groupby("from")
                .agg({"value": "count", "to": lambda x: ",".join(x)})
                .reset_index()
            )

            new_nodes_to = group_edges_to[group_edges_to["value"] >= min_group]

            if not new_nodes_to.empty:
                new_nodes_to["label"] = new_nodes_to.apply(
                    lambda row: f'{row["value"]} usuarios', axis=1
                )
                new_nodes_to["title"] = new_nodes_to.apply(
                    lambda row: f'Se agruparon: {row["value"]} usuarios', axis=1
                )
                new_nodes_to["image"] = "group_to"
                new_nodes_to["id"] = new_nodes_to.apply(
                    lambda row: f'{row["from"]}_a_{row["value"]}', axis=1
                )

                for _, row in new_nodes_to.iterrows():
                    edge_remove = row["to"].split(",")
                    nodes = nodes[~nodes["id"].isin(edge_remove)]
                    edges = edges[~edges["to"].isin(edge_remove)]

                new_nodes_to = new_nodes_to.drop(columns=["from", "to"])
                nodes = pd.concat([nodes, new_nodes_to])
                try:
                    nodes = nodes.drop(columns=["index"])
                except Exception:
                    pass

                new_edges = nodes[nodes["image"].eq("group_to")]
                new_edges["from"] = new_edges["id"].apply(
                    lambda index: index.split("_")[0]
                )
                new_edges = new_edges.rename(columns={"id": "to"})
                new_edges = new_edges[["from", "to", "value"]]
                edges = pd.concat([edges, new_edges])

            # apply max_nodes filter
            if max_nodes:
                nodes = nodes.sort_values("value", ascending=False)
                nodes = nodes.head(max_nodes)

                edges = edges[edges["from"].isin(nodes["id"])]
                edges = edges[edges["to"].isin(nodes["id"])]

            return nodes, edges

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            nodes = pd.DataFrame(columns=["id", "label", "value", "title", "image"])
            edges = pd.DataFrame(columns=["from", "to", "value"])
            return nodes, edges

    def accounts(self, engagement_rate="engagement_rate_by_post"):
        """
        This function computes the number of times an account is in the input DataFrame and its
        average engagement rate.

        Parameters
        ----------
        engagement_rate:
            type: str
            Determines the column of engagement rate for the computations,
            default='engagement_rate_by_post'

        Returns
        -------
        DataFrames
        """

        METHOD_NAME = "accounts"

        try:
            df_tweets_out = deepcopy(
                self.df_tweets_in[["screen_name", engagement_rate]]
            )
            df_tweets_out["posts_count"] = 1
            df_tweets_out = (
                df_tweets_out.groupby("screen_name")
                .agg({"posts_count": "count", engagement_rate: "mean"})
                .reset_index()
            )

            return df_tweets_out.sort_values("posts_count", ascending=False)

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(columns=["screen_name", engagement_rate, "posts_count"])



import sys
from copy import deepcopy

import pandas as pd

from ..common import general_utils
from ..common.nlp_utils import CleanText, Features, Polarity

ERR_SYS = "\nSystem error: "


class PolarityDistributionIG:
    """
    This class computes the polarity of the texts.
    """

    def __init__(self, df_comments, groups):
        """
        This method stores the input DataFrames and checks that they are not empty'.

        Parameters
        ----------
        df_comments:
            type: DataFrame
            Information of the instagram comments.
            This Pandas DataFrame must have columns 'post_owner_id', 'post_owner_username', 'text'.
        groups:
            type: dict
            Maps the groups (client, competition, archetype, trends) to the
            corresponding page ids for each group.
        """

        METHOD_NAME = "__init__"
        df_comments_full = deepcopy(df_comments)
        self.df_comments_full = df_comments_full
        self.groups = groups

        try:
            if df_comments.empty:
                print("Warning: input data DataFrame is empty.")
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            self.df_comments_full = pd.DataFrame(columns=[""])

    def get_polarity(self):
        """
        This method cleans the text in the comments and get its polarity.

        """

        METHOD_NAME = "get_polarity"
        try:

            # cleaning text:
            if "processed_text" not in self.df_comments_full.keys():
                self.df_comments_full["processed_text"] = self.df_comments_full[
                    "text"
                ].apply(
                    lambda msg: CleanText(msg).process_text(
                        mentions=True, hashtags=True, links=True, spec_chars=True
                    )
                )

            # drop empty comments
            self.df_comments_full = self.df_comments_full.dropna(
                subset=["processed_text"]
            )
            self.df_comments_full = self.df_comments_full.drop(
                self.df_comments_full[
                    self.df_comments_full["processed_text"] == ""
                ].index
            )

            # getting the polarity of the clean text
            if (
                "polarity" not in self.df_comments_full.keys()
                or None in self.df_comments_full.polarity.values
            ):
                self.df_comments_full = Polarity().polarity(self.df_comments_full)
                df_comments = self.df_comments_full
                return df_comments
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            self.df_comments_full["processed_text"] = ""
            self.df_comments_full["polarity"] = ""

    def grouped_polarities(self, group_by="group"):
        """
        This method get the text's polarity using the method get polarity. It then groups the texts and their polarity. It returns a data frame with number of texts in each polarity category for each group.

        Parameters
        ----------
        group_by:
            type: string
            takes four possible values: 'account' when the texts are grouped by individual instagram account, 'group' when the texts are grouped by the pre-defined (in the df_groups data frame) categories, 'all' when only one group contains all the analysed texts, and 'STTM_group' when the grouping is done using the clustering categories from an sttm algorithm. if no grouping information is found the default is "all"

        Returns
        df_groupedpolarity:
        Pandas DataFrame with with number of texts in each polarity category for each group.
        ----------
        """
        METHOD_NAME = "grouped_polarities"

        if (
            "polarity" not in self.df_comments_full.keys()
            or None in self.df_comments_full.polarity.values
        ):
            self.get_polarity()

        try:
            if "post_owner_id" in self.df_comments_full.keys():
                self.df_comments_full["group"] = self.df_comments_full[
                    "post_owner_id"
                ].apply(lambda pid: general_utils.get_group(pid, self.groups))
            else:
                self.df_comments_full["group"] = "no group"
                print(
                    "Warning: in Polarity_distibution_IG.grouped polarities no 'post_owner_id' column in DataFrame"
                )

            if "post_owner_username" not in self.df_comments_full.keys():
                print(
                    "Warning: in Polarity_distibution_IG.grouped polarities no 'post_owner_username' column in DataFrame"
                )
                self.df_comments_full["post_owner_username"] = "no name"

            self.df_comments_full["all"] = "all groups"

        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            if "group" not in self.df_comments_full.keys():
                self.df_comments_full["group"] = "no group"
            if "post_owner_username" not in self.df_comments_full.keys():
                self.df_comments_full["post_owner_username"] = "no name"
            self.df_comments_full["all"] = "all groups"

        try:
            if group_by == "account":
                df_groupedpolarity = (
                    self.df_comments_full.groupby(
                        ["post_owner_id", "post_owner_username", "group"]
                    )["polarity"]
                    .value_counts()
                    .unstack()
                    .fillna(0)
                )
                df_groupedpolarity["sentiment"] = df_groupedpolarity.to_dict(
                    orient="records"
                )
                df_groupedpolarity = df_groupedpolarity.reset_index().rename(
                    columns={
                        "post_owner_id": "_object_id",
                        "post_owner_username": "_object_name",
                    }
                )
                return df_groupedpolarity[
                    ["_object_id", "_object_name", "group", "sentiment"]
                ]
            elif group_by == "group":
                df_groupedpolarity = (
                    self.df_comments_full.groupby("group")["polarity"]
                    .value_counts()
                    .unstack()
                    .fillna(0)
                )
                df_groupedpolarity["sentiment"] = df_groupedpolarity.to_dict(
                    orient="records"
                )
                return df_groupedpolarity.reset_index()[["group", "sentiment"]]
            elif group_by == "all":
                df_groupedpolarity = (
                    self.df_comments_full.groupby("all")["polarity"]
                    .value_counts()
                    .unstack()
                    .fillna(0)
                )
                df_groupedpolarity["sentiment"] = df_groupedpolarity.to_dict(
                    orient="records"
                )
                return df_groupedpolarity.reset_index()[["all", "sentiment"]]
            else:
                print(
                    f"Warning: {group_by} Invalid parameter value for parameter group_by, grouping by all"
                )
                df_groupedpolarity = (
                    self.df_comments_full.groupby("all")["polarity"]
                    .value_counts()
                    .unstack()
                    .fillna(0)
                )
                df_groupedpolarity["sentiment"] = df_groupedpolarity.to_dict(
                    orient="records"
                )
                return df_groupedpolarity.reset_index()[["all", "sentiment"]]

        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            if group_by == "account":
                df_groupedpolarity = pd.DataFrame(
                    columns=["_object_id", "_object_name", "group", "sentiment"]
                )
            elif group_by == "group":
                df_groupedpolarity = pd.DataFrame(columns=["group", "sentiment"])
            elif group_by == "all":
                df_groupedpolarity = pd.DataFrame(columns=["all", "sentiment"])
            else:
                print(
                    f"Warning: {group_by} Invalid parameter value for parameter group_by, grouping by all"
                )
                df_groupedpolarity = pd.DataFrame(columns=["all", "sentiment"])
            return df_groupedpolarity



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr 23 10:03:01 2020

@author: oscarprecensia
"""

import re as reg
from copy import deepcopy
from sys import exc_info

import pandas as pd

pd.options.mode.chained_assignment = None

COL_MED = ["date_published", "title", "text", "url", "source"]
ERR_SYS = "system error: "


def create_regex(list_terms):
    """
    This class extract @name from a list

    Parameters
    ----------
    list_terms : TYPE list
        DESCRIPTION. list with the text to apply

    Returns
    -------
    pat : TYPE list
        DESCRIPTION. list without @name

    """
    pattern = r"\b@*" + r"\b|\b@*".join(list_terms) + r"\b"
    pat = reg.compile(pattern)
    return pat


class MediaPres:
    def __init__(self, dfm, df_count, brand, auto_n=True):
        """Digital media presence
        This function calculates digital media presence, comparing it with the
        weight of each digital media

        Parameters
        ----------
        dfm : TYPE dataframe
            DESCRIPTION. Contains the information collected from digital media
        df_count : TYPE dataframe
            DESCRIPTION. Contains the media information to consult
        count : string
            DESCRIPTION. brand analyze
        auto_n : TYPE bolean
            DESCRIPTION. remove mentions of the same account if true

        Returns
        -------
        None.

        """

        self.df_m = dfm
        self.df_count = df_count
        self.brand = brand
        self.auto_n = auto_n

    def count_pres(self):
        """Digital count media presence
        calculates digital media presence, only number of appearances

        Returns
        -------
        df_med_c
        TYPE dataframe
            DESCRIPTION. each count number of appearances in digital media

        """

        df_all = self.df_m
        df_c = self.df_count
        auto_n = self.auto_n
        df_empty_c = pd.DataFrame(columns=["media", "mentions"])
        method_name = "count_pres"

        try:
            df_w = df_all[COL_MED]
            df_w = deepcopy(df_w)
            df_w.date_published = pd.to_datetime(df_w.date_published)
            df_w.text = df_w.text.apply(lambda x: str(x))
            df_w.text = df_w.text.apply(lambda x: x.lower())
            df_m = deepcopy(df_w)

            for idd, row in df_c.set_index("nombre").iterrows():
                if idd:
                    account_terms_regex = create_regex(
                        [t.strip() for t in row.terminos.split(",")]
                    )
                    df_m[f"{idd}"] = df_m["text"].str.contains(
                        account_terms_regex, regex=True
                    )

            df_m["auto"] = 0

            if auto_n:
                col_names = df_m.columns[5:-1]
                for col in col_names:
                    temp_1 = df_m[col]
                    for index in range(len(df_m)):
                        if (temp_1[index] == True) & (df_m.source[index] == col):
                            df_m.auto.iloc[index] = 1
                df_m2 = df_m[df_m.auto == 0]
                df_m2 = df_m2.reset_index(drop=True)
            else:
                df_m2 = df_m

            df_med_c = df_m2.drop(
                columns=["date_published", "title", "text", "auto"]
            ).groupby("source")
            df_med_c = df_med_c.sum().T

            df_med_c = pd.DataFrame(df_med_c.T.sum())
            df_med_c = df_med_c.reset_index()
            df_med_c = df_med_c.rename(columns={0: "mentions", "index": "media"})
            df_med_c = df_med_c.sort_values("mentions", ascending=False)
            df_med_c = df_med_c.reset_index(drop=True)

        except KeyError as e_1:
            print(e_1)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_c = df_empty_c

        except AttributeError as e_2:
            print(e_2)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_c = df_empty_c

        except Exception as e_3:
            print(e_3)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_c = df_empty_c

        return df_med_c

    def url_pres(self):
        """Digital url media presence
        shows the url where the brand appears

        Returns
        -------
        df_med_url
        TYPE dataframe
            DESCRIPTION. url where the brand appears

        """

        df_all = self.df_m
        df_c = self.df_count
        brand = self.brand
        auto_n = self.auto_n
        df_empty_url = pd.DataFrame(
            columns=["date_published", "source", "title", "url"]
        )
        method_name = "url_pres"

        try:
            df_w = df_all[COL_MED]
            df_w = deepcopy(df_w)
            df_w.date_published = pd.to_datetime(df_w.date_published)

            df_w.text = df_w.text.apply(lambda x: str(x))
            df_w.text = df_w.text.apply(lambda x: x.lower())

            df_m = deepcopy(df_w)
            for idd, row in df_c.set_index("nombre").iterrows():
                if idd:
                    account_terms_regex = create_regex(
                        [t.strip() for t in row.terminos.split(",")]
                    )
                    df_m[f"{idd}"] = df_m["text"].str.contains(
                        account_terms_regex, regex=True
                    )

            df_m["auto"] = 0

            if auto_n:
                col_names = df_m.columns[5:-1]
                for col in col_names:
                    for index in range(len(df_m)):
                        temp_1 = df_m[col]
                        if (temp_1[index] == True) & (df_m.source[index] == col):
                            df_m.auto.iloc[index] = 1
                df_m2 = df_m[df_m.auto == 0]
                df_m2 = df_m2.reset_index(drop=True)
            else:
                df_m2 = df_m

            df_med_url = df_m2[["date_published", "source", "title", "url", brand]]
            df_med_url = df_med_url[df_med_url[brand] == True]
            df_med_url = df_med_url.drop(columns=brand)
            df_med_url = df_med_url.reset_index(drop=True)

        except KeyError as e_1:
            print(e_1)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_url = df_empty_url

        except AttributeError as e_2:
            print(e_2)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_url = df_empty_url

        except Exception as e_3:
            print(e_3)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_med_url = df_empty_url

        return df_med_url



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Aug 27 08:37:50 2020

@author: oscar
"""

from copy import deepcopy

import matplotlib.pyplot as plt
import pandas as pd
from textblob import TextBlob

# from wj_analysis.common.nlp_utils import Polarity as pol_dist
pd.options.mode.chained_assignment = None

FOLDER = "/home/oscar/Labs/Data_social_media/Pruebas PWJ-772 digital-social/Media/"
COLUMN = "text"
WORDS = ["rappi"]

DF_ORG = pd.read_csv(FOLDER + "df_media.csv", index_col=0)

DF_ORG.date_published = pd.to_datetime(DF_ORG.date_published)

# DF_ORG.text = DF_ORG.text.apply(lambda x: str(x))
# DF_ORG.text = DF_ORG.text.apply(lambda x: x.lower())

DF_ORG[COLUMN] = DF_ORG[COLUMN].apply(lambda x: str(x))
DF_ORG[COLUMN] = DF_ORG[COLUMN].apply(lambda x: x.lower())


def words_filter(df_w, words):
    # filtra la palabra objetivo
    DF_M = deepcopy(df_w)
    DF_M["word"] = None

    for i in range(len(DF_M)):
        for word in words:
            word = word.lower()
            if str(DF_M.text.iloc[i]).find(str(word)) > 0:
                DF_M.word.iloc[i] = "ok"
            else:
                continue

    DF_M_FILTER = DF_M[DF_M.word == "ok"]
    return DF_M_FILTER


DF_FILTER_1 = words_filter(df_w=DF_ORG, words=WORDS)


def find_point(df_p, carac="."):
    # filtra por puntos o el caracter escogido
    DF_P = deepcopy(df_p)
    Position = []
    for i in range(len(df_p)):
        pos = []
        last = 0
        text_c = str(DF_P.text.iloc[i])
        while text_c.find(carac, last) != -1:
            last = text_c.find(carac, last) + 1
            pos.append(last)
        Position.append(pos)
    return Position


DF_FILTER_1["Pos"] = find_point(df_p=DF_FILTER_1, carac=".")

for i in range(len(DF_FILTER_1)):
    plt.plot(
        range(len(DF_FILTER_1.Pos.iloc[i])), DF_FILTER_1.Pos.iloc[i], "o-", label=str(i)
    )
plt.title("Longitud de los textos palabra " + str(WORDS[0]))
plt.legend()
plt.show()

# parte entre puntos
text = 3
num_caract = 50
text_split = []
first = 0
phrases = []
for i in DF_FILTER_1.Pos.iloc[text]:
    p = DF_FILTER_1.text.iloc[text][first:i]
    long = i - first
    phrases.append(long)
    if long > num_caract:
        text_split.append(p)
    else:
        continue
    first = i

DF_POL = pd.DataFrame(text_split)
DF_POL["title"] = DF_FILTER_1.title.iloc[text]
DF_POL = DF_POL.rename(columns={0: "text"})

pol = []
for i in range(len(DF_POL)):
    get_pol = TextBlob(DF_POL.text.iloc[i])
    polarity = get_pol.polarity
    pol.append(polarity)

# pol = pol_dist().polarity(df_text=text_split)

plt.plot(range(len(pol)), pol, c="pink")
for i in range(len(pol)):
    plt.scatter(i, pol[i], s=len(text_split[i]) / 2, c=len(text_split[i]), alpha=0.8)
plt.title(str(DF_FILTER_1.title.iloc[text]))
plt.xlabel("numero de la frase")
plt.ylabel("Polaridad")
plt.grid()
plt.show()



import sys
from copy import deepcopy
from datetime import timedelta

import pandas as pd

from ..common import general_utils

ERR_SYS = "\nSystem error: "


class Traceability:
    def __init__(self, df_pages, groups):
        """
        This function computes the dataframe 'df_fan_count' with the columns
        'fan_count', 'page_id', 'name' 'group' and 'date'.

        Parameters
        ----------
        df_pages:
            type: DataFrame
            this Pandas DataFrame must have columns
            'fan_count', 'page_id' and 'created_at'.
        groups:
            type: dict
            Maps the groups (client, competition, archetype, trends) to the
            corresponding page ids for each group.
        """

        METHOD_NAME = "__init__"

        OUTPUT_COLUMNS = [
            "ac_followers_count",
            "twitter_id",
            "screen_name",
            "group",
            "date",
        ]

        if len(df_pages) > 0:
            try:
                df_fan_count = deepcopy(
                    df_pages[
                        ["ac_followers_count", "twitter_id", "screen_name", "date"]
                    ]
                )

                df_fan_count["date"] = pd.to_datetime(df_fan_count["date"]) - timedelta(
                    hours=5
                )
                df_fan_count["date"] = df_fan_count["date"].apply(lambda d: d.date())

                df_fan_count = (
                    df_fan_count.groupby(["twitter_id", "screen_name", "date"])
                    .last()
                    .reset_index()
                )

                df_fan_count["group"] = df_fan_count["twitter_id"].apply(
                    lambda tid: general_utils.get_group(tid, groups)
                )

                self.df_fan_count = df_fan_count

            except Exception as e:
                exception_type = sys.exc_info()[0]
                print(ERR_SYS + str(exception_type))
                print(e)
                print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
                self.df_fan_count = pd.DataFrame(columns=OUTPUT_COLUMNS)

        else:
            print("Warning: The DataFrame is empty. It cannot be computed.")
            self.df_fan_count = pd.DataFrame(columns=OUTPUT_COLUMNS)

    def absolute(self):
        """
        This method returns a DataFrame with the number of followers for each account
        for every day.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "absolute"

        df_fan_count = self.df_fan_count

        df_fan_count = df_fan_count.sort_values(
            ["group", "twitter_id", "screen_name", "date"]
        )
        df_fan_count = df_fan_count.rename(columns={"ac_followers_count": "value"})

        try:
            return df_fan_count[
                ["group", "twitter_id", "screen_name", "date", "value"]
            ].dropna(subset=["value"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=["group", "twitter_id", "screen_name", "date", "value"]
            )

    def differences(self):
        """
        This method returns a DataFrame with the number of new followers for each account
        for every day.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "differences"

        df_fan_count = self.df_fan_count
        try:
            dfs = []
            for tid in set(df_fan_count.twitter_id):
                df_tmp = df_fan_count[df_fan_count.twitter_id.eq(tid)].sort_values(
                    "date"
                )
                df_tmp["value"] = df_tmp["ac_followers_count"].diff()
                dfs.append(df_tmp)

            df_fan_count_diff = pd.concat(dfs).sort_values(
                ["group", "twitter_id", "screen_name", "date"]
            )

            return df_fan_count_diff[
                ["group", "twitter_id", "screen_name", "date", "value"]
            ].dropna(subset=["value"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=["group", "twitter_id", "screen_name", "date", "value"]
            )

    def percentage(self):
        """
        This method returns a DataFrame with the percentual change of followers for each account
        for every day.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "percentage"

        df_fan_count = self.df_fan_count
        try:
            dfs = []
            for tid in set(df_fan_count.twitter_id):
                df_tmp = df_fan_count[df_fan_count.twitter_id.eq(tid)].sort_values(
                    "date"
                )
                df_tmp["value"] = 100.0 * df_tmp["ac_followers_count"].pct_change()
                dfs.append(df_tmp)

            df_fan_count_perc = pd.concat(dfs).sort_values(
                ["group", "twitter_id", "screen_name", "date"]
            )

            return df_fan_count_perc[
                ["group", "twitter_id", "screen_name", "date", "value"]
            ].dropna(subset=["value"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=["group", "twitter_id", "screen_name", "date", "value"]
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Sep 02 17:22:09 2020, update Feb 12 10:56:00 2021 Oscar Riojas

@author: Joan-Felipe Mendoza
"""
from sys import exc_info

ERR_SYS = "System error: "


def get_series(df_input, item_name, item_col, metric, agg_cols=None, agg_func="mean"):
    """
    get_series

    Parameters
    ----------
    df_input : TYPE dataframe
        DESCRIPTION. input dataframe
    item_name : TYPE str
        DESCRIPTION.
    item_col : TYPE str
        DESCRIPTION.
    metric : TYPE str
        DESCRIPTION. metric transformation
    agg_cols : TYPE, optional
        DESCRIPTION. The default is None.
    agg_func : TYPE, optional
        DESCRIPTION. The default is 'mean'.

    Returns
    -------
    df_out : TYPE
        DESCRIPTION.

    """
    df_out = df_input.copy()
    df_out = df_out[df_out[item_col] == item_name]
    if agg_cols is not None:
        df_out = (
            df_out[agg_cols + [item_col, metric]]
            .groupby(agg_cols + [item_col])
            .agg({metric: agg_func})
        )
    return df_out


def remove_outliers(df_input, metric, method="qtl", n_sigma=3):
    """
    remove_outliers

    Parameters
    ----------
    df_input : TYPE dataframe
        DESCRIPTION. input dataframe
    metric : TYPE
        DESCRIPTION.
    method : TYPE, optional
        DESCRIPTION. The default is 'qtl'.
    n_sigma : TYPE, optional
        DESCRIPTION. The default is 3.

    Returns
    -------
    df_out : TYPE
        DESCRIPTION.

    """
    df_out = df_input.copy()
    pmin = df_out[metric].min()
    pmax = df_out[metric].max()
    df_out["outlier"] = 0
    if method == "qtl":
        q1 = df_out[metric].quantile(0.25)
        q3 = df_out[metric].quantile(0.75)
        irc = q3 - q1
        olb = max(pmin, q1 - n_sigma * irc)
        ohb = min(pmax, q3 + n_sigma * irc)
    if method == "std":
        avg = df_out[metric].mean()
        stdev = df_out[metric].std()
        olb = max(pmin, avg - n_sigma * stdev)
        ohb = min(pmax, avg + n_sigma * stdev)
    df_out.loc[df_out[metric] < olb, "outlier"] = 1
    df_out.loc[df_out[metric] > ohb, "outlier"] = 1
    df_out = df_out[df_out["outlier"] == 0]
    df_out = df_out.drop(columns=["outlier"])
    return df_out


def drop_data(df_input, metric, dr=0.01, dm="highest"):
    """
    drop_data

    Parameters
    ----------
    df_input : TYPE dataframe
        DESCRIPTION. input dataframe
    metric : TYPE
        DESCRIPTION.
    dr : TYPE, optional
        DESCRIPTION. The default is 0.01.
    dm : TYPE, optional
        DESCRIPTION. The default is 'highest'.

    Returns
    -------
    df_out : TYPE
        DESCRIPTION.

    """
    df_out = df_input.copy()
    olb, ohb = df_out[metric].min(), df_out[metric].max()
    if dm == "both":
        olb = df_out[metric].quantile(dr / 2, interpolation="midpoint")
        ohb = df_out[metric].quantile(1 - dr / 2, interpolation="midpoint")
    if dm == "lowest":
        olb = df_out[metric].quantile(dr, interpolation="midpoint")
    if dm == "highest":
        ohb = df_out[metric].quantile(1 - dr, interpolation="midpoint")
    df_out["outlier"] = 0
    df_out.loc[df_out[metric] < olb, "outlier"] = 1
    df_out.loc[df_out[metric] > ohb, "outlier"] = 1
    df_out = df_out[df_out["outlier"] == 0]
    df_out = df_out.drop(columns=["outlier"])
    return df_out


def get_boundaries(
    df_s, method="std", p_window=28, width_bound=1, metric="engagement_rate_by_day"
):
    """
    get_boundaries

    Parameters
    ----------
    df_s : TYPE
        DESCRIPTION.
    method : TYPE, optional
        DESCRIPTION. The default is 'std'.
    p_window : TYPE, optional
        DESCRIPTION. The default is 28.
    width_bound : TYPE, optional
        DESCRIPTION. The default is 1.
    metric : TYPE, optional
        DESCRIPTION. The default is 'engagement_rate_by_day'.

    Returns
    -------
    df_out : TYPE
        DESCRIPTION.

    """
    df_out = df_s.copy()  # Copy DF just for safety
    nrows = df_s.shape[0]
    if p_window < 5:
        p_window = 5
    if nrows > p_window:
        df_out["min"] = df_out[metric].rolling(p_window).min()  # Get MIN rolling column
        df_out["max"] = df_out[metric].rolling(p_window).max()  # Get MAX rolling column
    else:
        df_out["min"] = df_out[metric].min()  # Get MIN column
        df_out["max"] = df_out[metric].max()  # Get MAX column

    if method == "qtl":
        # Get QUANTILES rolling columns
        if nrows > p_window:
            df_out["q1"] = (
                df_out[metric]
                .rolling(p_window)
                .quantile(0.25, interpolation="midpoint")
            )
            df_out["q2"] = (
                df_out[metric]
                .rolling(p_window)
                .quantile(0.50, interpolation="midpoint")
            )
            df_out["q3"] = (
                df_out[metric]
                .rolling(p_window)
                .quantile(0.75, interpolation="midpoint")
            )
        else:
            df_out["q1"] = df_out[metric].quantile(0.25, interpolation="midpoint")
            df_out["q2"] = df_out[metric].quantile(0.50, interpolation="midpoint")
            df_out["q3"] = df_out[metric].quantile(0.75, interpolation="midpoint")
        df_out[["q1", "q2", "q3", "min", "max"]] = df_out[
            ["q1", "q2", "q3", "min", "max"]
        ].fillna(method="bfill")
        # df_out = df_out.dropna()
        # Get BOUNDARIES
        df_out["icr"] = df_out["q3"] - df_out["q1"]
        df_out["outer_lb"] = df_out["q1"] - 2 * width_bound * df_out["icr"]
        df_out["lb"] = df_out["q1"] - 1 * width_bound * df_out["icr"]
        df_out["hb"] = df_out["q3"] + 1 * width_bound * df_out["icr"]
        df_out["outer_hb"] = df_out["q3"] + 2 * width_bound * df_out["icr"]

    if method == "std":
        if nrows > p_window:
            df_out["mavg"] = (
                df_out[metric].rolling(p_window).mean()
            )  # Get MEAN rolling column
            df_out["mstd"] = (
                df_out[metric].rolling(p_window).std()
            )  # Get DEVIATION rolling column
        else:
            df_out["mavg"] = df_out[metric].mean()  # Get MEAN column
            df_out["mstd"] = df_out[metric].std()  # Get DEVIATION column
        df_out[["mavg", "mstd", "min", "max"]] = df_out[
            ["mavg", "mstd", "min", "max"]
        ].fillna(method="bfill")
        # df_out = df_out.dropna()
        # Get BOUNDARIES
        df_out["outer_lb"] = df_out["mavg"] - 2 * df_out["mstd"]
        df_out["lb"] = df_out["mavg"] - 1 * df_out["mstd"]
        df_out["hb"] = df_out["mavg"] + 1 * df_out["mstd"]
        df_out["outer_hb"] = df_out["mavg"] + 2 * df_out["mstd"]

    # Remove inconsistent boundary values
    df_out["outer_lb"] = df_out[["min", "outer_lb"]].max(axis=1)
    df_out["lb"] = df_out[["min", "lb"]].max(axis=1)
    df_out["hb"] = df_out[["max", "hb"]].min(axis=1)
    df_out["outer_hb"] = df_out[["max", "outer_hb"]].min(axis=1)
    df_out = df_out.drop(columns=["min", "max"])
    # Categorize each point within a boundary
    df_out["boundary"] = 0
    df_out.loc[df_out[metric] < df_out["lb"], "boundary"] = -1
    df_out.loc[df_out[metric] < df_out["outer_lb"], "boundary"] = -2
    df_out.loc[df_out[metric] > df_out["hb"], "boundary"] = 1
    df_out.loc[df_out[metric] > df_out["outer_hb"], "boundary"] = 2

    return df_out


class MetricCategorization:
    def __init__(self, df_s, metric, item_col, flag=0):
        """
        These functions deliver dataframes with categorization of metrics

        Parameters
        ----------
        df_s:
            type: Pandas DataFrame
            Pre-calculated dataframe containing all the columns required to do the expected
            categorization
        metric:
            type: str
            Name of the column in df_s containing the metric
        item_col:
            type: str
            Name of the column in df_s containing the items (i.e.: groups or brands)
        """
        self.df_s = df_s
        self.metric = metric
        self.item_col = item_col
        self.flag = flag

    def categorize(
        self, agg_cols=None, method="std", p_window=28, wb=1, dr=0, dm="highest"
    ):
        """
        Delivers a dataframe with boundary calculations and metric categorization in all rows,
        for each element of the item_col column, grouped by the item_col and agg_cols columns
        (optional).
        Parameters
        ----------
        agg_cols:
            type: list
            Names of the columns in df_s that are used as grouping elements.
            It should contain some column that is date.
        method:
            type: str
            Method to be applied to calculate the boundaries
        p_window:
            type: int
            Number of previous points to calculate the boundaries
        wb:
            type: float
            Width of boundary intervals (for a normal distribution, 1 = 1σ)
        dr:
            type: float
            Percentage of the data to be removed (between 0 and 1)
        dm:
            type: string
            Method to remove data ('lowest', 'highest' or 'both')
        Returns
            type: Pandas DataFrame
            Dataframe containing boundary calculations and metric categorization in all rows.
        -------
        """
        method_name = "categorize"
        self.df_s = self.df_s.reset_index(drop=True)

        try:
            df_woo = drop_data(self.df_s, self.metric, dr, dm)
            item_names = df_woo[self.item_col].unique().tolist()  # get item names
            # Get time series for each element in item_names
            df_ts = get_series(
                df_woo, item_names[0], self.item_col, self.metric, agg_cols
            )
            df_out = get_boundaries(df_ts, method, p_window, wb, self.metric)
            for item_name in item_names[1:]:
                df_ts = get_series(
                    df_woo, item_name, self.item_col, self.metric, agg_cols
                )
                df_temp = get_boundaries(df_ts, method, p_window, wb, self.metric)
                df_out = df_out.append(df_temp)
            rel_ef = []
            for row in range(len(df_out)):
                if df_out["mavg"].iloc[row] == 0:
                    rel_ef.append(self.flag)
                else:
                    temp_1 = (
                        100
                        * (df_out[self.metric].iloc[row] - df_out["mavg"].iloc[row])
                        / df_out["mavg"].iloc[row]
                    )
                    rel_ef.append(temp_1)
            df_out["rel_" + self.metric] = rel_ef

            return df_out.reset_index(drop=True)

        except TypeError as e_1:
            print(e_1)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            return self.df_s
        except Exception as e_2:
            print(e_2)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            return self.df_s



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  19 17:50:42 2021

@author: oscar
"""
import unittest

import pandas as pd

from wj_analysis.common.nlp_utils import Polarity

pd.options.mode.chained_assignment = None

sample_data = 1000

FOLDER = "/home/oscar/Labs/Data_polarity/"

df_com_fb = pd.read_csv(
    FOLDER + "Facebook/facebook_lib_facebook_comments.csv", low_memory=False
)
df_com_ig = pd.read_csv(
    FOLDER + "Instagram/instagram_lib_comment.csv", low_memory=False
)
df_com_tw = pd.read_csv(FOLDER + "Twitter/twitter_lib_tweetreply.csv", low_memory=False)

print("".center(60, "="))
print("ut_polarity_only_fb.py")
print("comments = " + str(sample_data * 3))
print("".center(60, "="))


class Test(unittest.TestCase):
    def setUp(self):
        """
        Variables to use in tests

        Returns
        -------
        None.

        """

        self.polarity = Polarity()
        self.df_com_s_fb = df_com_fb.sample(n=sample_data)
        self.df_com_s_ig = df_com_ig.sample(n=sample_data)
        self.df_com_s_tw = df_com_tw.sample(n=sample_data)
        self.df_empty = pd.DataFrame(columns=self.df_com_s_tw.columns)

    def test_data_normal(self):
        """
        This test with data ok

        Returns
        -------
        None.

        """
        print("".center(60, "="))
        print("TEST_DATA_NORMAL...")
        df_fb_normal = self.df_com_s_fb
        df_ig_normal = self.df_com_s_ig
        df_tw_normal = self.df_com_s_tw

        self.polarity.polarity_only(df_text=df_fb_normal, column_text="message")
        self.polarity.polarity_only(df_text=df_ig_normal, column_text="text")
        self.polarity.polarity_only(df_text=df_tw_normal, column_text="text")

        # valida dataframe no vacio
        self.assertGreater(len(df_fb_normal), 0)
        self.assertGreater(len(df_ig_normal), 0)
        self.assertGreater(len(df_ig_normal), 0)

        # valida una columna adicional con la polaridad
        self.assertEqual(len(df_fb_normal.columns), 17)
        self.assertEqual(len(df_ig_normal.columns), 11)
        self.assertEqual(len(df_tw_normal.columns), 45)

        print("".center(60, "-"))

    def test_data_empty(self):
        """
        This test with data empty

        Returns
        -------
        None.

        """
        print("".center(60, "="))
        print("TEST_DATA_EMPTY...")
        df_empty = self.df_empty

        self.polarity.polarity_only(df_text=df_empty, column_text="text")

        self.assertEqual(len(df_empty), 0)

        print("".center(60, "-"))

    @unittest.expectedFailure
    def test_wrong_column(self):
        """
        This test with data 20% null

        Returns
        -------
        None.

        """
        print("".center(60, "="))
        print("TEST_NAN_DATA...")
        df_fb_nc = self.df_com_s_fb
        df_ig_nc = self.df_com_s_ig
        df_tw_nc = self.df_com_s_tw

        self.polarity.polarity_only(df_text=df_fb_nc, column_text="sdf")
        self.polarity.polarity_only(df_text=df_ig_nc, column_text="fg")
        self.polarity.polarity_only(df_text=df_tw_nc, column_text="mesdfgdsage")

        # valida dataframe vacio por error en la columna
        self.assertGreater(len(df_fb_nc), 0)
        self.assertGreater(len(df_ig_nc), 0)
        self.assertGreater(len(df_tw_nc), 0)

        print("".center(60, "-"))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 14 19:50:57 2020

@author: oscar
"""

from copy import deepcopy
from sys import exc_info

import pandas as pd

ERR_SYS = "System error: "


class EffectMomentsFB:
    def __init__(self, df_fb, trends=False, grouped=True):
        """Effective moments Facebook
        This class calculates the average of the effectiveness in one day,
        the average of the effectiveness per hour in a week
        and the effectiveness of the hours of a week for facebook

        Parameters
        ----------
        df_FB : TYPE, dataframe
            DESCRIPTION. with facebook accounts filtered by account or
            accounts, competitor and archetype, with calculated effectiveness
        trends : TYPE, bolean
            DESCRIPTION. if required trends in the data
        grouped : TYPE, bolean
            DESCRIPTION. if False it is grouped by brands, if True by groups

        Returns
        -------
        None.

        """
        self.df_org = df_fb
        self.trends = trends
        self.grouped = grouped

    def moments(self):
        """Moments effective Facebook
        This function calculates the average of the effectiveness in one day,
        the average of the effectiveness per hour,
        in a week and the effectiveness of the hours of a week for facebook

        Returns
        -------
        df_eff_day : TYPE dataframe
            DESCRIPTION. effectiveness hours day
        df_eff_hours : TYPE dataframe
            DESCRIPTION. effectiveness hour a week
        df_eff_days_hours : TYPE dataframe
            DESCRIPTION. effectiveness days and hour with a week

        """

        column_post = [
            "page_id",
            "created_time",
            "group",
            "post_id",
            "engagement_rate_by_post",
            "page_name",
            "rel_engagement_rate_by_post",
        ]
        group = ["brand", "archetypes", "competitors"]

        method_name = "moments"

        if self.grouped is True:
            df_empty_day = pd.DataFrame(
                columns=[
                    "day",
                    "group",
                    "counts",
                    "engagement_rate_by_post",
                    "rel_engagement_rate_by_post",
                ]
            )
            df_empty_hours = pd.DataFrame(
                columns=[
                    "hour",
                    "group",
                    "counts",
                    "engagement_rate_by_post",
                    "rel_engagement_rate_by_post",
                ]
            )
            df_empty_days_week = pd.DataFrame(
                columns=[
                    "hour",
                    "day",
                    "group",
                    "counts",
                    "engagement_rate_by_post",
                    "rel_engagement_rate_by_post",
                ]
            )
        else:
            df_empty_day = pd.DataFrame(
                columns=[
                    "day",
                    "group",
                    "_object_name",
                    "counts",
                    "_object_id",
                    "engagement_rate_by_post",
                    "rel_engagement_rate_by_post",
                ]
            )
            df_empty_hours = pd.DataFrame(
                columns=[
                    "hour",
                    "group",
                    "_object_name",
                    "counts",
                    "_object_id",
                    "engagement_rate_by_post",
                    "rel_engagement_rate_by_post",
                ]
            )
            df_empty_days_week = pd.DataFrame(
                columns=[
                    "hour",
                    "day",
                    "group",
                    "counts",
                    "_object_name",
                    "_object_id",
                    "engagement_rate_by_post",
                    "rel_engagement_rate_by_post",
                ]
            )

        try:
            df_original = self.df_org[column_post]
            df_original = deepcopy(df_original)
            if df_original.isna().any().any():
                df_original = None

        except KeyError as e_1:
            print(e_1)
            error_1 = exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
        trends = self.trends
        grouped = self.grouped

        if trends:
            group.append("trends")

        try:
            df_original.created_time = pd.to_datetime(df_original.created_time)
            df_copy = deepcopy(df_original)

            df_copy["day"] = df_copy.created_time.apply(lambda x: x.weekday())
            df_copy["hour"] = df_copy.created_time.apply(lambda x: x.hour)
            df_copy["date"] = df_copy.created_time.apply(lambda x: x.date())
            df_copy["hour"] = df_copy.hour.apply(lambda x: str(x))
            df_copy["day"] = df_copy.day.apply(lambda x: str(x))

            df_filter = deepcopy(df_copy.reset_index())

            if grouped is True:
                grouped_id = "group"
            else:
                grouped_id = "page_id"

            df_eff_day = pd.DataFrame()
            for sel_group in group:
                temp_1 = df_filter[df_filter.group == sel_group]
                if grouped is True:
                    temp_1 = temp_1.groupby(["day", grouped_id], as_index=False).agg(
                        {"engagement_rate_by_post": "mean", "post_id": "count"}
                    )
                else:
                    temp_1 = temp_1.groupby(
                        ["day", "group", grouped_id], as_index=False
                    ).agg(
                        {
                            "engagement_rate_by_post": "mean",
                            "page_name": "last",
                            "post_id": "count",
                        }
                    )
                df_eff_day = pd.concat([df_eff_day, temp_1])

            df_eff_day = df_eff_day.sort_values(
                "engagement_rate_by_post", ascending=False
            ).reset_index()
            df_eff_day = df_eff_day.drop(columns="index")
            df_eff_day = df_eff_day.rename(
                columns={
                    "page_id": "_object_id",
                    "page_name": "_object_name",
                    "post_id": "counts",
                }
            )

            df_eff_hours = pd.DataFrame()
            for sel_group in group:
                temp_1 = df_filter[df_filter.group == sel_group]
                if grouped is True:
                    temp_1 = temp_1.groupby(["hour", grouped_id], as_index=False).agg(
                        {"engagement_rate_by_post": "mean", "post_id": "count"}
                    )
                else:
                    temp_1 = temp_1.groupby(
                        ["hour", "group", grouped_id], as_index=False
                    ).agg(
                        {
                            "engagement_rate_by_post": "mean",
                            "page_name": "last",
                            "post_id": "count",
                        }
                    )
                df_eff_hours = pd.concat([df_eff_hours, temp_1])

            df_eff_hours = df_eff_hours.sort_values(
                "engagement_rate_by_post", ascending=False
            ).reset_index()
            df_eff_hours = df_eff_hours.drop(columns="index")
            df_eff_hours = df_eff_hours.rename(
                columns={
                    "page_id": "_object_id",
                    "page_name": "_object_name",
                    "post_id": "counts",
                }
            )

            df_eff_days_hours = pd.DataFrame()
            for group in group:
                temp_1 = df_filter[df_filter.group == group]
                if grouped is True:
                    temp_1 = temp_1.groupby(
                        ["hour", "day", grouped_id], as_index=False
                    ).agg({"engagement_rate_by_post": "mean", "post_id": "count"})
                else:
                    temp_1 = temp_1.groupby(
                        ["hour", "day", "group", grouped_id], as_index=False
                    ).agg(
                        {
                            "engagement_rate_by_post": "mean",
                            "page_name": "last",
                            "post_id": "count",
                        }
                    )
                df_eff_days_hours = pd.concat([df_eff_days_hours, temp_1])

            df_eff_days_hours = df_eff_days_hours.sort_values(
                "engagement_rate_by_post", ascending=False
            ).reset_index()
            df_eff_days_hours = df_eff_days_hours.drop(columns="index")
            df_eff_days_hours = df_eff_days_hours.rename(
                columns={
                    "page_id": "_object_id",
                    "page_name": "_object_name",
                    "post_id": "counts",
                }
            )

        except UnboundLocalError as e_2:
            print(e_2)
            error_1 = exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_eff_day = df_empty_day
            df_eff_hours = df_empty_hours
            df_eff_days_hours = df_empty_days_week

        except KeyError as e_3:
            print(e_3)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_eff_day = df_empty_day
            df_eff_hours = df_empty_hours
            df_eff_days_hours = df_empty_days_week

        except Exception as e_4:
            print(e_4)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            df_eff_day = df_empty_day
            df_eff_hours = df_empty_hours
            df_eff_days_hours = df_empty_days_week

        return df_eff_day, df_eff_hours, df_eff_days_hours



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jul 30 14:04:09 2020

@author: oscar
"""

from copy import deepcopy
from sys import exc_info

import pandas as pd

NEG = "page_negative_feedback_by_type_unique"
ERR_SYS = "System error: "


class NegFeedback:
    def __init__(self, df_r, lim_sdt=True, lim_sdt_p=1, lim=0):
        """
        Negative Feedback
        this function generates an alarm the moment its value goes out of limits

        Parameters
        ----------
        df_r : TYPE dataframe
            DESCRIPTION. dataframe with information from facebook insights, should
            have page_negative_feedback_by_type_unique
        lim_sdt : TYPE, boolean
            DESCRIPTION. The default is true. if the limit is the standard deviation
        lim_sdt_p : TYPE, float
            DESCRIPTION. The default is 1. standard deviation adjustment
        lim : TYPE, float
            DESCRIPTION. The default is 0. manual upper limit value if lim_sdt is false

        Returns
        -------
        None.

        """

        self.df_r = df_r
        self.lim_sdt = lim_sdt
        self.lim_sdt_p = lim_sdt_p
        self.lim = lim

    def neg_feedback(self):
        """
        Negative Feedback
        this function generates an alarm the moment its value goes out of limits

        Returns
        -------
        perf_n : TYPE string
            DESCRIPTION. higher: alarm,
            ok: expected performance
        val_neg : TYPE int
            DESCRIPTION. value page_negative_feedback_by_type_unique

        """

        df_r = deepcopy(self.df_r)
        lim_sdt = self.lim_sdt
        lim_sdt_p = self.lim_sdt_p
        lim = self.lim

        var_empty = None
        method_name = "neg_feedback"

        df_r = df_r[df_r.name == NEG]
        df_r = df_r[df_r.period == "day"]
        df_r.end_time = pd.to_datetime(df_r.end_time)

        try:
            df_group = df_r.groupby(["end_time", "name"], as_index=False).sum()
            df_mod = df_group[df_group.end_time != max(df_r.end_time)]
            df_neg = df_group[df_group.end_time == max(df_r.end_time)]
            val_neg = df_neg.value.iloc[0]

            if lim_sdt:
                lim_sup = df_mod.value.std() * lim_sdt_p
            else:
                lim_sup = lim

            if val_neg >= lim_sup:
                perf_n = "higher"
            else:
                perf_n = "ok"

        except AttributeError as e_1:
            print(e_1)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            perf_n = var_empty
            val_neg = var_empty

        except IndexError as e_2:
            print(e_2)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            perf_n = var_empty
            val_neg = var_empty

        except Exception as e_3:
            print(e_3)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            perf_n = var_empty
            val_neg = var_empty

        return perf_n, val_neg



import numpy as np
import pandas as pd


def get_group(pid, groups):
    """
    This functions get the group of the corresponding id of the page.

    Parameters
    ----------
    pid:
        type: str
        page_id to conpute the group for.
    groups:
        type: dict
        Maps the group to the page ids.

    Returns
    -------
    str
    """

    group_out = "no_group"
    try:
        for group in groups:
            if pid in groups[group]:
                group_out = group
    except Exception:
        pass
    return group_out


def discretize(df, from_column, to_column=None, n_groups=10):
    """
    This functions discretizes de column 'from_column' into the column 'to_column' in the
    dataframe df.

    Parameters
    ----------
    n_groups:
        type: str
        Number of groups to split the column 'from_column'.
        default=10

    Returns
    -------
    None
    """
    if from_column in df.keys():
        if to_column:
            df[to_column] = pd.cut(
                df[from_column], n_groups, labels=range(1, n_groups + 1)
            )
        else:
            df[f"disc_{from_column}"] = pd.cut(
                df[from_column], n_groups, labels=range(1, n_groups + 1)
            )
    else:
        raise RuntimeError(
            f"{from_column} is not a column available of input DataFrame."
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Apr 30 15:15:45 2021

@author: oscar
"""

import traceback
from copy import deepcopy
from sys import exc_info

import pandas as pd

pd.options.mode.chained_assignment = None


ERR_SYS = "system error: "

COLUMNS_DATA = [
    "headline",
    "text",
    "content",
    "url",
    "image",
    "created_at",
    "source.name",
    "word",
]


def words_filter(df_w, words, column, n_f):
    """
    words_filter

    Parameters
    ----------
    df_w : TYPE dataframe
        DESCRIPTION. dataframe with data from google-news
    words : TYPE array
        DESCRIPTION. array with the words to search
    column : TYPE, string
        DESCRIPTION. column to search word
    n_f : TYPE, string
        DESCRIPTION. return value word is not found
    Returns
    -------
    df_m_filter : TYPE dataframe
        DESCRIPTION. dataframe with column 'word'

    """

    df_m_filter_empty = pd.DataFrame(columns=COLUMNS_DATA)
    method_name = "GNews_words_filter"
    df_m = deepcopy(df_w)
    df_m["word"] = "-"

    if len(df_m) != 0:

        try:
            for i in range(len(df_m)):
                for word in words:
                    if str(df_m[column].iloc[i]).find(str(word)) > 0:
                        df_m.word.iloc[i] = str(word)
                    else:
                        continue

            df_m_filter = df_m[df_m.word != "-"]

            if len(df_m_filter) == 0:
                data_c = []
                row_c = 0
                while row_c < len(COLUMNS_DATA):
                    data_c.append(n_f)
                    row_c += 1
                df_m_filter = pd.DataFrame([data_c], columns=COLUMNS_DATA)

        except KeyError as e_1:
            print("==========================================================")
            print(e_1)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Method: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_m_filter = df_m_filter_empty

        except AttributeError as e_2:
            print("==========================================================")
            print(e_2)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Method: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_m_filter = df_m_filter_empty

        except Exception as e_3:
            print("==========================================================")
            print(e_3)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Method: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_m_filter = df_m_filter_empty
    else:
        print("Dataframe news empty!")
        df_m_filter = df_m_filter_empty

    return df_m_filter


def prepare_data(df_p, words, column, n_f):
    """
    prepare_data

    Parameters
    ----------
    df_p : TYPE dataframe
        DESCRIPTION. dataframe to search words
    words : TYPE array
        DESCRIPTION. array with the words to search
    column : TYPE, string
        DESCRIPTION. column to search word
    n_f : TYPE, string
        DESCRIPTION. return value word is not found

    Returns
    -------
    df_filter : TYPE dataframe
        DESCRIPTION. dataframe with column 'word'

    """
    df_filter_empty = pd.DataFrame(
        columns=[
            "headline",
            "text",
            "content",
            "url",
            "image",
            "created_at",
            "source",
            "word",
        ]
    )
    method_name = "GNews_prepare_data"

    try:
        df_p["created_at"] = pd.to_datetime(
            df_p["created_at"], format="%Y-%m-%d %H:%M:%S.%f"
        )
        df_filter = words_filter(df_w=df_p, words=words, column=column, n_f=n_f)
        df_filter = df_filter.drop_duplicates()

    except KeyError as e_1:
        print("==========================================================")
        print(e_1)
        print("==========================================================")
        print(ERR_SYS + str(exc_info()[0]))
        print("==========================================================")
        print(f"Method: {method_name}")
        print("==========================================================")
        traceback.print_exc()

        df_filter = df_filter_empty

    except AttributeError as e_2:
        print("==========================================================")
        print(e_2)
        print("==========================================================")
        print(ERR_SYS + str(exc_info()[0]))
        print("==========================================================")
        print(f"Method: {method_name}")
        print("==========================================================")
        traceback.print_exc()

        df_filter = df_filter_empty

    except Exception as e_3:
        print("==========================================================")
        print(e_3)
        print("==========================================================")
        print(ERR_SYS + str(exc_info()[0]))
        print("==========================================================")
        print(f"Method: {method_name}")
        print("==========================================================")
        traceback.print_exc()

        df_filter = df_filter_empty

    return df_filter


class GNews:
    """
    This class search words in Gnews data
    """

    def __init__(self, df_g, query, column="text", n_f="not_found"):
        """


        Parameters
        ----------
        df_g : TYPE dataframe
            DESCRIPTION. dataframe with columns:
            'headline', 'text', 'content', 'url', 'image', 'created_at', 'source.name'
        query : TYPE array
            DESCRIPTION. array with the words to search
        column : TYPE, optional
            DESCRIPTION. The default is 'text' column to search word
        n_f : TYPE, optional
            DESCRIPTION. The default is 'not_found' return value word is not found

        Returns
        -------
        None.

        """

        self.df_g = df_g
        self.query = query
        self.n_f = n_f
        self.column = column
        self.df_filter = prepare_data(df_p=df_g, words=query, column=column, n_f=n_f)

    def count_pres(self):
        """
        count_pres

        Returns
        -------
        df_filter_c : TYPE dataframe
            DESCRIPTION. dataframe with words and count by word

        """
        df_filter_c = self.df_filter
        df_filter_c_empty = pd.DataFrame(columns=["word", "count"])
        method_name = "count_pres"

        try:

            df_filter_c = df_filter_c.groupby("word", as_index=False).agg(
                {"headline": "count"}
            )
            df_filter_c = df_filter_c.rename(columns={"headline": "count"})

        except KeyError as e_1:
            print("==========================================================")
            print(e_1)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_filter_c = df_filter_c_empty

        except AttributeError as e_2:
            print("==========================================================")
            print(e_2)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_filter_c = df_filter_c_empty

        except Exception as e_3:
            print("==========================================================")
            print(e_3)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_filter_c = df_filter_c_empty

        return df_filter_c

    def url_pres(self):
        """
        url_pres

        Returns
        -------
        df_filter_u : TYPE dataframe
            DESCRIPTION. dataframe with word match and columns 'created_at',
            'source.name', 'word', 'url', 'headline'

        """
        df_filter_u = self.df_filter
        df_filter_u_empty = pd.DataFrame(
            columns=["created_at", "source.name", "word", "url", "headline"]
        )
        method_name = "url_pres"

        try:

            df_filter_u = df_filter_u.groupby("url", as_index=False).agg(
                {
                    "word": "last",
                    "created_at": "last",
                    "headline": "last",
                    "source.name": "last",
                }
            )
            df_filter_u = df_filter_u.sort_values("created_at").reset_index(drop=True)

        except KeyError as e_1:
            print("==========================================================")
            print(e_1)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_filter_u = df_filter_u_empty

        except AttributeError as e_2:
            print("==========================================================")
            print(e_2)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_filter_u = df_filter_u_empty

        except Exception as e_3:
            print("==========================================================")
            print(e_3)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_filter_u = df_filter_u_empty

        return df_filter_u



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May 17 14:32:33 2021

@author: oscar
"""

import random
import unittest

import pandas as pd

from wj_analysis.reach_fb_ig.reach_fb_ig import ReachFaceInst

FOLDER = "/home/oscar/MEGA/W&J/Insigths_PWJ-1827/Data/"
FILES_INS = ["ig_insights_wyj.csv", "ig_insights_wyj.csv"]
FILES_POST = ["instagram_lib_posts_WYJ.csv", "instagram_lib_posts_WYJ.csv"]

SOCIAL_N = "ig"

BRAND = random.sample(range(len(FILES_POST)), 1)

COLUMNS_POST = ["owner_username", "shortcode", "date_local", "url"]

DFP = FILES_POST[int(BRAND[0])]
DFI = FILES_INS[int(BRAND[0])]

DF_POST = pd.read_csv(FOLDER + DFP, usecols=COLUMNS_POST)
DF_INS = pd.read_csv(FOLDER + DFI)

print("================================================")
print("ut_reach_IG.py")
print("post = " + str(len(DF_INS)))
print("================================================")


class Test(unittest.TestCase):
    def setUp(self):
        """
        Variables to use in tests

        Returns
        -------
        None.

        """

        self.df_post = DF_POST
        self.df_ins = DF_INS
        self.social_n = SOCIAL_N

    def test_data_normal(self):
        """
        This test with data ok

        Returns
        -------
        None.

        """
        print("TEST_DATA_NORMAL...")

        df_post = self.df_post
        df_ins = self.df_ins
        social_n = self.social_n

        df_reach = ReachFaceInst(
            df_post=df_post, df_ins=df_ins, social_n=social_n
        ).reach()

        self.assertGreater(len(df_reach), 0)

        print("================================================")

    def test_data_empty(self):
        """
        This test with data empty

        Returns
        -------
        None.

        """
        print("TEST_DATA_EMPTY...")

        df_post = self.df_post
        df_ins = self.df_ins
        social_n = self.social_n

        df_post_empty = pd.DataFrame(columns=df_post.columns)

        df_ins_empty = pd.DataFrame(columns=df_ins.columns)

        df_reach_empty = ReachFaceInst(
            df_post=df_post_empty, df_ins=df_ins, social_n=social_n, url=True
        ).reach()
        df_reach_empty_post = ReachFaceInst(
            df_post=df_post_empty, df_ins=df_ins, social_n=social_n, url=True
        ).reach()
        df_reach_empty_ins = ReachFaceInst(
            df_post=df_post, df_ins=df_ins_empty, social_n=social_n, url=True
        ).reach()

        self.assertEqual(len(df_reach_empty), 0)
        self.assertEqual(len(df_reach_empty_post), 0)
        self.assertEqual(len(df_reach_empty_ins), 0)

        print("================================================")

    def test_nan_data(self):
        """
        This test with data 20% nan

        Returns
        -------
        None.

        """
        print("TEST_NAN_DATA...")

        df_post = self.df_post
        df_ins = self.df_ins
        social_n = self.social_n

        df_post_nan = df_post
        column_sample = random.sample(list(df_post_nan.columns), 1)
        array_ef = random.sample(list(range(len(df_post_nan))), len(df_post_nan) // 5)
        for k in array_ef:
            df_post_nan["shortcode"].iloc[k] = float("nan")
            df_post_nan[column_sample[0]].iloc[k] = float("nan")

        df_ins_nan = df_ins
        column_sample = random.sample(list(df_ins_nan.columns), 1)
        array_ef = random.sample(list(range(len(df_ins_nan))), len(df_ins_nan) // 5)
        for k in array_ef:
            df_ins_nan["shortcode"].iloc[k] = float("nan")
            df_ins_nan[column_sample[0]].iloc[k] = float("nan")

        df_reach_nan = ReachFaceInst(
            df_post=df_post_nan, df_ins=df_ins_nan, social_n=social_n
        ).reach()
        df_reach_nan_post = ReachFaceInst(
            df_post=df_post_nan, df_ins=df_ins, social_n=social_n
        ).reach()
        df_reach_nan_ins = ReachFaceInst(
            df_post=df_post, df_ins=df_ins_nan, social_n=social_n
        ).reach()

        self.assertGreater(len(df_reach_nan), 0)
        self.assertGreater(len(df_reach_nan_post), 0)
        self.assertGreater(len(df_reach_nan_ins), 0)

        print("================================================")

    def test_no_column(self):
        """
        This test three columns are removed

        Returns
        -------
        None.

        """
        print("TEST_NO_COLUMNS...")

        df_post = self.df_post
        df_ins = self.df_ins
        social_n = self.social_n

        df_post_no_col = df_post
        df_post_no_col = df_post_no_col.drop(columns=["shortcode"])

        df_ins_no_col = df_ins
        df_ins_no_col = df_ins_no_col.drop(columns=["shortcode"])

        df_reach_no_col = ReachFaceInst(
            df_post=df_post_no_col, df_ins=df_ins_no_col, social_n=social_n
        ).reach()
        df_reach_no_col_post = ReachFaceInst(
            df_post=df_post_no_col, df_ins=df_ins, social_n=social_n
        ).reach()
        df_reach_no_col_ins = ReachFaceInst(
            df_post=df_post, df_ins=df_ins_no_col, social_n=social_n
        ).reach()

        self.assertEqual(len(df_reach_no_col), 0)
        self.assertEqual(len(df_reach_no_col_post), 0)
        self.assertEqual(len(df_reach_no_col_ins), 0)


if __name__ == "__main__":
    unittest.main()



import json
import os
import re
import sys

import numpy as np
import pandas as pd
import requests
from numpy import argmax, exp, log
from numpy.random import multinomial

ERR_SYS = "\nSystem error: "
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
URL_MS_MODELOS = os.getenv("URL_MS_MODELOS", None)


def delete_file_dataframe(file_to_send, file_send):
    try:
        file_send["model_json"].close()
        os.remove(file_to_send) if os.path.exists(file_to_send) else None
    except OSError as error:
        print("Error con eliminacion archivo" + error)
    except Exception as e:
        print("Error con eliminacion archivo" + e)


class CleanText:
    def __init__(self, input_txt, verbose=False):
        """
        This functions validates if the input of the class is a string.

        Parameters
        ----------
        input_txt:
            type: str
            String to clean.
        mentions:
            verbose: bool
            If True prints when the input is not a string.
        """
        METHOD_NAME = "init"
        if type(input_txt) == str:
            self.input_txt = input_txt
        else:
            print(f'WARNING: Input {input_txt} is not a string. Default set to "".')
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            self.input_txt = ""

    def process_text(
        self, rts=False, mentions=False, hashtags=False, links=False, spec_chars=False
    ):
        """
        This functions cleans the input text.

        Parameters
        ----------
        rts:
            type: bool
            If True the patterns associated with retweets are removed
            from the text, default=False.
        mentions:
            type: bool
            If True the mentions are removed from the text, default=False.
        hashtags:
            type: bool
            If True the hashtags are removed from the text, default=False.
        links:
            type: bool
            If True the patterns associated with links (urls) are removed
            from the text, default=False.
        spec_chars:
            type: bool
            If True all special characters (except accents, # and @) are removed
            from the text, default=False.

        Returns
        -------
        str
        """

        input_txt = self.input_txt
        if rts:
            rt_pattern = re.compile(r"^(?:RT|rt) \@[a-zA-Z0-9\-\_]+\b")
            input_txt = re.sub(rt_pattern, "", input_txt)
        if mentions:
            mention_pattern = re.compile(r"\@[a-zA-Z0-9\-\_]+\b")
            input_txt = re.sub(mention_pattern, "", input_txt)
        else:
            # procect '@' signs of being removed in spec_chars
            input_txt = input_txt.replace("@", "xxatsignxx")
        if hashtags:
            hashtag_pattern = re.compile(r"\#[a-zA-Z0-9\-\_]+\b")
            input_txt = re.sub(hashtag_pattern, "", input_txt)
        else:
            # procect '#' signs to being removed in spec_chars
            input_txt = input_txt.replace("#", "xxhashtagsignxx")
        if links:
            link_pattern = re.compile(r"\bhttps:.+\b")
            input_txt = re.sub(link_pattern, "", input_txt)
            link_pattern = re.compile(r"\bhttp:.+\b")
            input_txt = re.sub(link_pattern, "", input_txt)
        if spec_chars:
            input_txt = re.sub(r"[^a-zA-Z\u00C0-\u00FF ]", " ", input_txt)
        input_txt = input_txt.split()
        output_txt = " ".join(input_txt)
        output_txt = output_txt.replace("xxatsignxx", "@")
        output_txt = output_txt.replace("xxhashtagsignxx", "#")
        return output_txt


class Features:
    def __init__(self):
        """
        This functions loads the spacy model for spanish.

        """
        # self.nlp_model = Language().from_disk(path_models + 'pipeline')

        self.bad_words = ["de", "que", "la", "los", "el", "las"]
        # def remove_names(self, rem_names=False, names_list=[]):
        # if(rem_names ==True):
        #    self.bad_words = self.bad_words + names_list

    def pos_tags(self, input_txt):
        """
        This functions get the features of the words in the input_txt parameter.

        Parameters
        ----------
        input_txt:
            type: str
            String to get the features from.

        Returns
        -------
        dict: keys -> tokens, lemmas and part of speech tags.
        i
        """
        METHOD_NAME = "pos_tags"
        self.input_txt = input_txt
        if type(input_txt) != str:
            print("ERROR: Input is not a string.")
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            self.input_txt = ""
        self.bad_words

        try:
            out_dict = send_to_model(
                data={"value": input_txt},
                ms_models_path=URL_MS_MODELOS + "/models_reg/features/pos_tags",
            )
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            out_dict = {"words": [], "lemmas": [], "pos_tags": []}

        return out_dict


class Polarity(object):
    def __init__(self):
        self.polscore = [-1.0, -0.5, 0, 0.5, 1.0]

    def tokenize_texts(self, text_list, max_length=128):
        try:
            model_name = "polarity"
            model_type = "tokenize_texts"
            encoded_dict = df_to_formatted(
                text_list, URL_MS_MODELOS, model_name, model_type
            )
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            raise e
        input_ids_b = encoded_dict["input_ids"]
        attention_masks_b = encoded_dict["attention_mask"]

        return (input_ids_b, attention_masks_b)

    def forward_model(self, data):
        try:
            model_name = "polarity"
            model_type = "tokenize_texts"
            (out,) = df_to_formatted(data, URL_MS_MODELOS, model_name, model_type)
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            raise e

        sentiment = list(out.argmax(1).numpy())

        return sentiment

    def batch_forward(self, text_list, batch_size, max_length):

        num_texts = len(self.df_text)
        batch_idx = slice(0, num_texts, batch_size)

        polarity = []
        for i in range(num_texts // batch_size):
            batch_idx = slice(batch_size * i, batch_size * (i + 1), 1)
            (input_ids_b, attention_masks_b) = self.tokenize_texts(
                text_list[batch_idx], max_length=max_length
            )
            sentiment_list = self.forward_model((input_ids_b, attention_masks_b))

            polarity.extend(sentiment_list)

        if num_texts % batch_size:
            batch_idx = slice(batch_size * (num_texts // batch_size), num_texts, 1)
            (input_ids_b, attention_masks_b) = self.tokenize_texts(
                text_list[batch_idx], max_length=max_length
            )
            sentiment_list = self.forward_model((input_ids_b, attention_masks_b))

            polarity.extend(sentiment_list)

        return polarity

    def batch_polarity(
        self, df_text, text_column_name="processed_text", batch_size=2, max_length=128
    ):
        """
        This function returns the polarity of an input texts procesed by batches.

        Parameters
        ----------
        df_text:
            type: str, list of strings, pandas DataFrame with a text column.
            clean text for which the polarity is obtained

        text_column_name (optional):
            type: str, name of the text column in the pandas DataFrame.

        batch_size (optional):
            type: int, number of text to process at the same time by doing a forward in the model.
            default: 20

        Returns
        -------
        pandas DataFrame with a 'polarity' column with the polarity values.
        (warning, this method returns -100 if input is not a list of strings or pandas DataFrame.)

        """

        METHOD_NAME = "batch_polarity"

        if isinstance(df_text, list) and all(isinstance(text, str) for text in df_text):
            df_text = pd.DataFrame(columns=["processed_text"], data=df_text)

        if isinstance(df_text, pd.DataFrame):
            try:
                df_text["polarity"] = self.batch_forward(
                    list(df_text[text_column_name]),
                    batch_size=batch_size,
                    max_length=max_length,
                )
                dict_polscore = {idx: score for idx, score in enumerate(self.polscore)}
                df_text["polarity"] = df_text["polarity"].map(dict_polscore)

            except Exception as e:
                print(e)
                error_1 = sys.exc_info()[0]
                print(ERR_SYS + str(error_1))
                print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
                df_text["polarity"] = ""
            return df_text

        elif isinstance(df_text, str):
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            print(
                "Must recieve a list of string of pandas DataFrame with strings. returning -100 ..."
            )
            return -100

        else:
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            print(
                "Calling polarity method for a non-string variable. returning -100 ..."
            )
            polarity = -100
            return polarity

    def polarity(self, df_text, text_column_name="processed_text"):
        """
        This function returns the polarity of an input text(s).

        Parameters
        ----------
        df_text:
            type: str, list of strings, pandas DataFrame with a text column.
            clean text for which the polarity is obtained

        text_column_name (optional):
            type: str, name of the text column in the pandas DataFrame.

        Returns
        -------
        polarity value or pandas DataFrame with a 'polarity' column with the polarity values. (warning, this method returns -100 if input is not a string or if input text is too long.)

        """
        try:
            model_name = "polarity"
            model_type = "polarity"
            polarity = df_to_formatted(df_text, URL_MS_MODELOS, model_name, model_type)
            return polarity
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            raise e

    def polarity_only(self, df_text, column_text="message"):
        """
        This function returns the polarity of an dataframe


        Parameters
        ----------
        df_text : TYPE dataframe
            DESCRIPTION.dataframe, with the column polarity to be calculated
        column_text : TYPE, string
            DESCRIPTION. text column.with the polarity to be calculated
            for Facebook = message
            for Instagram = text
            for Twitter = text

        Returns
        -------
        polarity value or pandas DataFrame with a 'polarity' column with the polarity values
        """
        try:
            model_name = "polarity"
            model_type = "polarity_only"
            polarity = df_to_formatted(df_text, URL_MS_MODELOS, model_name, model_type)
            return polarity
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            raise e


class STTM(object):
    def __init__(self, df_data, text_column_name="tokenized_text"):
        """
        This function stores the input DataFrame.

        Parameters
        ----------
        df_data:
            type: DataFrame
            This Pandas DataFrame must have the column 'tokenized text'.

        """

        self.df_data = df_data
        self.text_column_name = text_column_name

    def sttm_model(self, K=20, alpha=0.1, beta=0.2, n_iters=100):
        """
        This function classifies the tokenized texts into groups of similar texts. It creates the 'sttm_group' column with the number of the group that the text was classified into.

        Parameters
        ----------
        K:
            type: int
            Number of initial groups that the texts are classified into.
        alpha:
            type: float
            Parameter of the sttm model .
        beta:
            type: float
            Parameter of the sttm model.
        n_iters:
            type: int
            Number of iterations of the sttm model.

        Returns
        ----------
        df_data:
            DataFrame with the 'sttm_group' column

        """

        try:
            model_name = "sttm"
            model_type = "sttm_model"
            self.df_data = df_to_formatted(
                self.df_data, URL_MS_MODELOS, model_name, model_type
            )
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            raise e
        return self.df_data


def send_to_model(data, ms_models_path, ensure_ascii=True):
    """
    Auxiliar function used to send data to models microservice

    Parameters
    ----------
    data :
        Request body
    ms_models : [type]
        Path of models microservice
    """

    response = requests.post(url=ms_models_path, data=json.dumps(data))

    if response.ok:
        return response.json()
    else:
        raise RuntimeError(f"Models microservice error {response.status_code}")


def df_to_formatted(df, url_model, model_name, model_type):
    ms_models_path = url_model + "/models_reg/" + model_name + "/" + model_type

    if model_name == "polarity":
        df["polarity"] = df["processed_text"].apply(
            lambda txt: send_to_model({"value": txt}, ms_models_path)
        )
    elif model_name == "sttm":

        response = send_to_model(
            data=df[["id", "tokenized_text"]].to_dict(orient="records"),
            ms_models_path=ms_models_path,
            ensure_ascii=False,
        )

        df = pd.merge(df, pd.DataFrame(response, columns=["id", "sttm_group"]), on="id")
    else:
        raise NotImplementedError()

    return df



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat May 15 19:27:23 2021

@author: oscar
"""

import ast
import traceback
from sys import exc_info

import pandas as pd

pd.options.mode.chained_assignment = None


ERR_SYS = "system error: "

POSTS_COLUMNS_FB = ["post_from", "post_id", "created_time", "permalink_url"]
EMPTY_COLUMNS_FB = [
    "post_id",
    "created_time",
    "name",
    "post_impressions",
    "post_impressions_unique",
    "post_impressions_paid",
    "post_impressions_paid_unique",
    "post_impressions_fan",
    "post_impressions_fan_unique",
    "post_impressions_fan_paid",
    "post_impressions_fan_paid_unique",
    "post_impressions_organic",
    "post_impressions_organic_unique",
    "post_impressions_viral",
    "post_impressions_viral_unique",
    "post_impressions_nonviral",
    "post_impressions_nonviral_unique",
    "post_impressions_by_story_type",
    "post_impressions_by_story_type_unique",
]

POSTS_COLUMNS_IG = ["owner_username", "shortcode", "date_local", "url"]
EMPTY_COLUMNS_IG = [
    "shortcode",
    "date_local",
    "name",
    "impressions",
    "reach",
    "engagement",
]


class ReachFaceInst:
    """
    This class match posts to reach
    """

    def __init__(self, df_post, df_ins, social_n="fb", url=False):
        """
        This method joins the information of publications with the information of insights,
        reads the information of several brands, in list format is important

        Parameters
        ----------
        df_post : TYPE dataframe
            DESCRIPTION. posts information
        df_ins : TYPE dataframe
            DESCRIPTION. insights information
        social_n : TYPE string
            DESCRIPTION. indicates social network, 'fb': Facebook, 'ig': Instagram
        url : TYPE string
            DESCRIPTION. is True if load post url

        Returns
        -------
        None.

        """

        self.df_post = df_post
        self.df_ins = df_ins
        self.social_n = social_n
        self.url = url

    def reach(self):
        """
        joint informtion posts and insights

        Returns
        -------
        df_reach : TYPE dataframe
            DESCRIPTION. dataframe whit reach, brand, date and post_id

        """

        df_post = self.df_post
        df_ins = self.df_ins
        social_n = self.social_n
        url = self.url
        method_name = "reach Facebook Instagram"

        try:

            if social_n == "fb":
                if url:
                    EMPTY_COLUMNS_FB.append("permalink_url")

                EMPTY_COLUMNS = EMPTY_COLUMNS_FB
                POSTS_COLUMNS = POSTS_COLUMNS_FB
                POST_FROM = "post_from"
                ON = "post_id"
                NAME = "name"

                df_reach_empty = pd.DataFrame(columns=EMPTY_COLUMNS)

                if df_post[POST_FROM].isna().values.any():
                    df_post = df_post[df_post[POST_FROM].notna()]

                brand_name = df_post[POST_FROM].apply(lambda x: ast.literal_eval(x))
                name = []
                for i in range(len(df_post)):
                    temp_3 = brand_name.iloc[i][NAME]
                    name.append(temp_3)

                df_post = df_post[POSTS_COLUMNS]
                df_post = df_post.drop(columns="post_from")
                df_post[NAME] = name

            elif social_n == "ig":
                if url:
                    EMPTY_COLUMNS_IG.append("url")

                df_post = df_post.rename(columns={"owner_username": "name"})
                EMPTY_COLUMNS = EMPTY_COLUMNS_IG
                POSTS_COLUMNS = POSTS_COLUMNS_IG
                POST_FROM = "shortcode"
                ON = "shortcode"
                NAME = "name"

                df_reach_empty = pd.DataFrame(columns=EMPTY_COLUMNS)

                if df_post[POST_FROM].isna().values.any():
                    df_post = df_post[df_post[POST_FROM].notna()]

            else:
                print(f"{social_n} is not a valid value")

            df_reach = pd.merge(df_post, df_ins, on=ON, how="outer")
            df_reach = df_reach[EMPTY_COLUMNS]
            df_reach = df_reach.dropna().reset_index(drop=True)

        except KeyError as e_1:
            print("==========================================================")
            print(e_1)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_reach = df_reach_empty

        except AttributeError as e_2:
            print("==========================================================")
            print(e_2)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_reach = df_reach_empty

        except Exception as e_3:
            print("==========================================================")
            print(e_3)
            print("==========================================================")
            print(ERR_SYS + str(exc_info()[0]))
            print("==========================================================")
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            print("==========================================================")
            traceback.print_exc()

            df_reach = df_reach_empty

        return df_reach



import json
import sys
import warnings
from copy import deepcopy

import pandas as pd

from ..common import general_utils
from ..common.nlp_utils import CleanText, Features

warnings.filterwarnings("ignore", "This pattern has match groups")
warnings.filterwarnings("ignore", category=DeprecationWarning)

ERR_SYS = "\nSystem error: "


class MostEffectiveContentIG:
    """
    This class computes the different atributes of the posts with
    higher engagement rate.
    """

    def __init__(self, df_posts, groups):
        """
        This method computes the DataFrame 'df_posts_full'
        which contains all the information of the posts, including column
        'group'.

        Parameters
        ----------
        df_posts:
            type: DataFrame
            Information of the posts.
            This Pandas DataFrame must have columns 'page_id',
            'page_name', 'message' and 'engagement_rate_by_post'.
        groups:
            type: dict
            Maps the groups (client, compentition, archetype, trends) to the
            corresponding page ids for each group.
        """

        METHOD_NAME = "__init__"

        POSTS_COLUMNS = [
            "owner_id",
            "owner_username",
            "shortcode",
            "caption",
            "typename",
            "caption_hashtags",
            "caption_mentions",
            "group",
            "engagement_rate_by_post",
            "rel_engagement_rate_by_post",
        ]

        OUTPUT_COLUMNS = [
            "owner_id",
            "owner_username",
            "shortcode",
            "caption",
            "typename",
            "caption_hashtags",
            "caption_mentions",
            "group",
            "engagement_rate_by_post",
            "rel_engagement_rate_by_post",
        ]

        self.features = None

        if len(df_posts) > 0:
            try:
                df_posts_full = deepcopy(df_posts[POSTS_COLUMNS])
                df_posts_full["owner_id"] = df_posts_full["owner_id"].apply(
                    lambda uid: str(uid)
                )

                df_posts_full["group"] = df_posts_full["owner_id"].apply(
                    lambda pid: general_utils.get_group(pid, groups)
                )

                self.df_posts_full = df_posts_full
                self.len_posts_full = len(self.df_posts_full)

            except Exception as e:
                exception_type = sys.exc_info()[0]
                print(ERR_SYS + str(exception_type))
                print(e)
                print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
                self.df_posts_full = pd.DataFrame(columns=OUTPUT_COLUMNS)
                self.len_posts_full = len(self.df_posts_full)

        else:
            print("Warning: The DataFrame is empty. It cannot be computed.")
            self.df_posts_full = pd.DataFrame(columns=OUTPUT_COLUMNS)
            self.len_posts_full = len(self.df_posts_full)

    def posts(self, n_most_eff=5, engagement_rate="engagement_rate_by_post", **kwargs):
        """
        This function computes the DataFrame 'df_most_eff_posts_ig'
        which contains the posts with higher engagement rate for the accounts selected.

        Parameters
        ----------
        n_most_eff:
            type: int
            Number of posts to show, default=5.
        engagement_rate:
            type: str
            Determines the column of engagement rate for the computations,
            default='engagement_rate_by_post'
        **kwargs:
            account_ids:
                type: list
                Ids of the accounts to look for.
                If [] takes all the posts in the DataFrame posts.
            account_names:
                type: list
                Name of the accounts to look for.
                If [] takes all the posts in the DataFrame posts.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "posts"

        if "account_ids" in kwargs.keys() and kwargs["account_ids"]:
            account_ids = kwargs["account_ids"]
            df_most_eff_posts_ig = self.df_posts_full[
                self.df_posts_full.owner_id.isin(account_ids)
            ][
                [
                    "owner_username",
                    "shortcode",
                    "caption",
                    engagement_rate,
                    "rel_" + engagement_rate,
                ]
            ]
        elif "account_names" in kwargs.keys() and kwargs["account_names"]:
            account_names = kwargs["account_names"]
            df_most_eff_posts_ig = self.df_posts_full[
                self.df_posts_full.owner_username.isin(account_names)
            ][
                [
                    "owner_username",
                    "shortcode",
                    "caption",
                    engagement_rate,
                    "rel_" + engagement_rate,
                ]
            ]
        else:
            df_most_eff_posts_ig = deepcopy(self.df_posts_full)[
                [
                    "owner_username",
                    "shortcode",
                    "caption",
                    engagement_rate,
                    "rel_" + engagement_rate,
                ]
            ]
        try:
            df_most_eff_posts_ig["url"] = df_most_eff_posts_ig["shortcode"].apply(
                lambda x: f"instagram.com/p/{x}"
            )
            df_most_eff_posts_ig = df_most_eff_posts_ig.sort_values(
                engagement_rate, ascending=False
            ).head(n_most_eff)

            return df_most_eff_posts_ig

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=[
                    "owner_username",
                    "shortcode",
                    "caption",
                    engagement_rate,
                    "rel_" + engagement_rate,
                    "url",
                ]
            )

    def posts_type(
        self,
        from_n_most_eff=None,
        engagement_rate="engagement_rate_by_post",
        grouped=True,
        **kwargs,
    ):
        """
        This function computes the DataFrame 'df_most_eff_posts_type_tw'
        which contains for each media type its proportion to the whole number of posts
        and its average engagement rate.

        Parameters
        ----------
        from_n_most_eff:
            type: int
            Number of posts to compute the ratios from, default=None means
            the computations is against all posts.
        engagement_rate:
            type: str
            Determines the column of engagement rate for the computations,
            default='engagement_rate_by_post'
        grouped:
            type: bool
            Determines if the output is returned grouped by group or account,
            default=True.
        **kwargs:
            account_ids:
                type: list
                Ids of the accounts to look for.
                If [] takes all the posts in the DataFrame posts.
            account_names:
                type: list
                Name of the accounts to look for.
                If [] takes all the posts in the DataFrame posts.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "posts_type"

        if not from_n_most_eff:
            from_n_most_eff = self.len_posts_full

        PAGE_COLUMNS = []
        if not grouped:
            PAGE_COLUMNS = ["owner_id", "owner_username"]

        if "account_ids" in kwargs.keys() and kwargs["account_ids"]:
            account_ids = kwargs["account_ids"]
            df_most_eff_posts_type_ig = self.df_posts_full[
                self.df_posts_full.owner_id.isin(account_ids)
            ][
                PAGE_COLUMNS
                + ["typename", "caption", engagement_rate, "rel_" + engagement_rate]
            ]
        elif "account_names" in kwargs.keys() and kwargs["account_names"]:
            account_names = kwargs["account_names"]
            df_most_eff_posts_type_ig = self.df_posts_full[
                self.df_posts_full.owner_username.isin(account_names)
            ][
                PAGE_COLUMNS
                + ["typename", "caption", engagement_rate, "rel_" + engagement_rate]
            ]
        else:
            df_most_eff_posts_type_ig = deepcopy(
                self.df_posts_full[
                    PAGE_COLUMNS
                    + ["typename", "caption", engagement_rate, "rel_" + engagement_rate]
                ]
            )

        try:
            df_most_eff_posts_type_ig = df_most_eff_posts_type_ig.sort_values(
                engagement_rate, ascending=False
            ).head(from_n_most_eff)

            df_most_eff_posts_type_ig["caption_count"] = 1
            df_most_eff_posts_type_ig = (
                df_most_eff_posts_type_ig[
                    PAGE_COLUMNS
                    + [
                        "typename",
                        "caption_count",
                        engagement_rate,
                        "rel_" + engagement_rate,
                    ]
                ]
                .groupby(PAGE_COLUMNS + ["typename"])
                .agg(
                    {
                        "caption_count": "count",
                        engagement_rate: "mean",
                        "rel_" + engagement_rate: "mean",
                    }
                )
            )
            df_most_eff_posts_type_ig = df_most_eff_posts_type_ig.rename(
                columns={
                    "caption_count": "counts",
                    engagement_rate: "avg_engagement_rate",
                    "rel_" + engagement_rate: "avg_rel_engagement_rate",
                }
            )
            if PAGE_COLUMNS:
                df_most_eff_posts_type_ig["percentage"] = (
                    df_most_eff_posts_type_ig["counts"]
                    .groupby(level=0)
                    .apply(lambda c: 100.0 * c / float(c.sum()))
                    .round(2)
                )
                df_most_eff_posts_type_ig = (
                    df_most_eff_posts_type_ig.reset_index().rename(
                        columns={
                            PAGE_COLUMNS[0]: "_object_id",
                            PAGE_COLUMNS[1]: "_object_name",
                        }
                    )
                )
                PAGE_COLUMNS = ["_object_id", "_object_name"]
            else:
                df_most_eff_posts_type_ig["percentage"] = (
                    100.0
                    * df_most_eff_posts_type_ig["counts"]
                    / df_most_eff_posts_type_ig["counts"].sum()
                ).round(2)
                df_most_eff_posts_type_ig = df_most_eff_posts_type_ig.reset_index()

            return df_most_eff_posts_type_ig.sort_values(
                PAGE_COLUMNS + ["avg_engagement_rate"], ascending=False
            )

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            if PAGE_COLUMNS:
                PAGE_COLUMNS = ["_object_id", "_object_name"]
            return pd.DataFrame(
                columns=PAGE_COLUMNS
                + [
                    "typename",
                    "counts",
                    "avg_engagement_rate",
                    "avg_rel_engagement_rate",
                    "percentage",
                ]
            )

    def construct_most_eff_objects(
        self,
        df_most_eff_objects_ig,
        objs,
        engagement_rate="engagement_rate_by_post",
        grouped=True,
        sort_column="engagement_rate_by_post",
    ):
        """
        This function computes the output DataFrame for hashtags or mentions
        with their respective count and engagement rate.

        Parameters
        ----------
        df_most_eff_objects_ig:
            type: DataFrame
            DataFrame with objects (hashtags or mentions) to extract.
        objs:
            type: str
            Objects (hashtags or mentions) to compute the DataFrame.
        engagement_rate:
            type: str
            Determines the column of engagement rate for the computations,
            default='engagement_rate_by_post'.
        grouped:
            type: bool
            Determines if the output is returned grouped by group or account,
            default=True.
        sort_column:
            Column to sort the output DataFrame,
            default='engagement_rate_by_post'.

        Returns
        -------
        DataFrames
        """

        METHOD_NAME = "construct_most_eff_objects"

        PAGE_COLUMNS = []
        if not grouped:
            PAGE_COLUMNS = ["owner_id", "owner_username"]

        try:
            objs_ig = []
            objs_eff_ig = []
            objs_rel_eff_ig = []
            objs_group_ig = []
            if PAGE_COLUMNS:
                objs_ids_ig = []
                objs_names_ig = []
            objs_counts_ig = []

            for _, row in df_most_eff_objects_ig.iterrows():
                objs_ig = objs_ig + row["caption_" + objs]
                objs_eff_ig = objs_eff_ig + [row[engagement_rate]] * len(
                    row["caption_" + objs]
                )
                objs_rel_eff_ig = objs_rel_eff_ig + [
                    row["rel_" + engagement_rate]
                ] * len(row["caption_" + objs])
                objs_group_ig = objs_group_ig + [row.group] * len(
                    row["caption_" + objs]
                )
                if PAGE_COLUMNS:
                    objs_ids_ig = objs_ids_ig + [row[PAGE_COLUMNS[0]]] * len(
                        row["caption_" + objs]
                    )
                    objs_names_ig = objs_names_ig + [row[PAGE_COLUMNS[1]]] * len(
                        row["caption_" + objs]
                    )
                objs_counts_ig = objs_counts_ig + [1] * len(row["caption_" + objs])

            most_eff_objs_ig = pd.DataFrame(
                {
                    objs[:-1]: objs_ig,
                    engagement_rate: objs_eff_ig,
                    "rel_" + engagement_rate: objs_rel_eff_ig,
                    "group": objs_group_ig,
                    f"{objs[:-1]}_count": objs_counts_ig,
                }
            )

            if PAGE_COLUMNS:
                most_eff_objs_ig[PAGE_COLUMNS[0]] = objs_ids_ig
                most_eff_objs_ig[PAGE_COLUMNS[1]] = objs_names_ig

            most_eff_objs_ig = (
                most_eff_objs_ig.groupby(PAGE_COLUMNS + ["group", objs[:-1]])
                .agg(
                    {
                        engagement_rate: "mean",
                        "rel_" + engagement_rate: "mean",
                        f"{objs[:-1]}_count": "sum",
                    }
                )
                .sort_values(PAGE_COLUMNS + [sort_column], ascending=False)
                .reset_index()
            )

            return most_eff_objs_ig

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            most_eff_objs_ig = pd.DataFrame(
                columns=PAGE_COLUMNS
                + [
                    objs[:-1],
                    engagement_rate,
                    "rel_" + engagement_rate,
                    "group",
                    f"{objs[:-1]}_count",
                ]
            )

            return most_eff_objs_ig

    def hashtags_mentions(
        self,
        compare_group="brand",
        from_n_most_eff=None,
        engagement_rate="engagement_rate_by_post",
        grouped=True,
        **kwargs,
    ):
        """
        This function computes the following DataFrames:
          - 'most_eff_hashtags_ig' and 'most_eff_mentions_ig'
            which contains the hastags and mentions with higher associated
            engagement rate for the accounts selected.

          - 'compare_hashtags_ig' and 'compare_mentions_ig'
            which contains the most used hastags and mentions for the account
            to compare with.

        Parameters
        ----------
        compare_group:
            type: str
            Name of the group to compare with. default='brand'.
        from_n_most_eff:
            type: int
            Number of posts to compute the ratios from, default=None means
            the computations is against all posts.
        engagement_rate:
            type: str
            Determines the column of engagement rate for the computations,
            default='engagement_rate_by_post'.
        grouped:
            type: bool
            Determines if the output is returned grouped by group or account,
            default=True.
        **kwargs:
            account_ids:
                type: list
                Ids of the accounts to look for.
                If [] takes all the posts in the DataFrame posts.
            account_names:
                type: list
                Name of the accounts to look for.
                If [] takes all the posts in the DataFrame posts.

        Returns
        -------
        Tuple of DataFrames
        """

        METHOD_NAME = "hashtags_mentions"

        if not from_n_most_eff:
            from_n_most_eff = self.len_posts_full

        PAGE_COLUMNS = []
        if not grouped:
            PAGE_COLUMNS = ["owner_id", "owner_username"]

        if "account_ids" in kwargs.keys() and kwargs["account_ids"]:
            account_ids = kwargs["account_ids"]
            df_most_eff_hashtags_mentions_ig = self.df_posts_full[
                self.df_posts_full.owner_id.isin(account_ids)
            ][
                PAGE_COLUMNS
                + [
                    "caption_hashtags",
                    "caption_mentions",
                    "group",
                    engagement_rate,
                    "rel_" + engagement_rate,
                ]
            ]
        elif "account_names" in kwargs.keys() and kwargs["account_names"]:
            account_names = kwargs["account_names"]
            df_most_eff_hashtags_mentions_ig = self.df_posts_full[
                self.df_posts_full.owner_username.isin(account_names)
            ][
                PAGE_COLUMNS
                + [
                    "caption_hashtags",
                    "caption_mentions",
                    "group",
                    engagement_rate,
                    "rel_" + engagement_rate,
                ]
            ]
        else:
            df_most_eff_hashtags_mentions_ig = deepcopy(
                self.df_posts_full[
                    PAGE_COLUMNS
                    + [
                        "caption_hashtags",
                        "caption_mentions",
                        "group",
                        engagement_rate,
                        "rel_" + engagement_rate,
                    ]
                ]
            )

        df_compare_hashtags_mentions_ig = self.df_posts_full[
            self.df_posts_full.group == compare_group
        ][
            PAGE_COLUMNS
            + [
                "caption_hashtags",
                "caption_mentions",
                "group",
                engagement_rate,
                "rel_" + engagement_rate,
            ]
        ]

        try:
            df_most_eff_hashtags_mentions_ig[
                "caption_hashtags"
            ] = df_most_eff_hashtags_mentions_ig["caption_hashtags"].apply(
                lambda hashtags: json.loads(hashtags)
            )
            df_most_eff_hashtags_mentions_ig[
                "caption_hashtags"
            ] = df_most_eff_hashtags_mentions_ig["caption_hashtags"].apply(
                lambda hashtags: hashtags if hashtags else None
            )

            df_most_eff_hashtags_mentions_ig[
                "caption_mentions"
            ] = df_most_eff_hashtags_mentions_ig["caption_mentions"].apply(
                lambda mentions: json.loads(mentions)
            )
            df_most_eff_hashtags_mentions_ig[
                "caption_mentions"
            ] = df_most_eff_hashtags_mentions_ig["caption_mentions"].apply(
                lambda mentions: mentions if mentions else None
            )

            df_compare_hashtags_mentions_ig[
                "caption_hashtags"
            ] = df_compare_hashtags_mentions_ig["caption_hashtags"].apply(
                lambda hashtags: json.loads(hashtags)
            )
            df_compare_hashtags_mentions_ig[
                "caption_hashtags"
            ] = df_compare_hashtags_mentions_ig["caption_hashtags"].apply(
                lambda hashtags: hashtags if hashtags else None
            )

            df_compare_hashtags_mentions_ig[
                "caption_mentions"
            ] = df_compare_hashtags_mentions_ig["caption_mentions"].apply(
                lambda mentions: json.loads(mentions)
            )
            df_compare_hashtags_mentions_ig[
                "caption_mentions"
            ] = df_compare_hashtags_mentions_ig["caption_mentions"].apply(
                lambda mentions: mentions if mentions else None
            )

            df_most_eff_hashtags_mentions_ig = (
                df_most_eff_hashtags_mentions_ig.sort_values(
                    engagement_rate, ascending=False
                ).head(from_n_most_eff)
            )
            df_compare_hashtags_mentions_ig = (
                df_compare_hashtags_mentions_ig.sort_values(
                    engagement_rate, ascending=False
                ).head(from_n_most_eff)
            )
            # Hashtags
            df_most_eff_hashtags_ig = df_most_eff_hashtags_mentions_ig[
                ~df_most_eff_hashtags_mentions_ig["caption_hashtags"].isna()
            ]
            most_eff_hashtags_ig = self.construct_most_eff_objects(
                df_most_eff_hashtags_ig, objs="hashtags", grouped=grouped
            )

            # Group to compare with
            df_compare_hashtags_ig = df_compare_hashtags_mentions_ig[
                ~df_compare_hashtags_mentions_ig["caption_hashtags"].isna()
            ]
            compare_hashtags_ig = self.construct_most_eff_objects(
                df_compare_hashtags_ig,
                objs="hashtags",
                grouped=grouped,
                sort_column="hashtag_count",
            )

            # Mentions
            df_most_eff_mentions_ig = df_most_eff_hashtags_mentions_ig[
                ~df_most_eff_hashtags_mentions_ig["caption_mentions"].isna()
            ]
            most_eff_mentions_ig = self.construct_most_eff_objects(
                df_most_eff_mentions_ig, objs="mentions", grouped=grouped
            )

            # Group to compare with
            df_compare_mentions_ig = df_compare_hashtags_mentions_ig[
                ~df_compare_hashtags_mentions_ig["caption_mentions"].isna()
            ]
            compare_mentions_ig = self.construct_most_eff_objects(
                df_compare_mentions_ig,
                objs="mentions",
                grouped=grouped,
                sort_column="mention_count",
            )

            if PAGE_COLUMNS:
                most_eff_hashtags_ig = most_eff_hashtags_ig.rename(
                    columns={
                        PAGE_COLUMNS[0]: "_object_id",
                        PAGE_COLUMNS[1]: "_object_name",
                    }
                )
                compare_hashtags_ig = compare_hashtags_ig.rename(
                    columns={
                        PAGE_COLUMNS[0]: "_object_id",
                        PAGE_COLUMNS[1]: "_object_name",
                    }
                )
                most_eff_mentions_ig = most_eff_mentions_ig.rename(
                    columns={
                        PAGE_COLUMNS[0]: "_object_id",
                        PAGE_COLUMNS[1]: "_object_name",
                    }
                )
                compare_mentions_ig = compare_mentions_ig.rename(
                    columns={
                        PAGE_COLUMNS[0]: "_object_id",
                        PAGE_COLUMNS[1]: "_object_name",
                    }
                )
                PAGE_COLUMNS = ["_object_id", "_object_name"]

            return (
                most_eff_hashtags_ig,
                compare_hashtags_ig,
                most_eff_mentions_ig,
                compare_mentions_ig,
            )

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            if PAGE_COLUMNS:
                PAGE_COLUMNS = ["_object_id", "_object_name"]
            most_eff_hashtags_ig = pd.DataFrame(
                columns=PAGE_COLUMNS
                + [
                    "hashtag",
                    engagement_rate,
                    "rel_" + engagement_rate,
                    "group",
                    "hashtag_count",
                ]
            )
            compare_hashtags_ig = pd.DataFrame(
                columns=PAGE_COLUMNS
                + [
                    "hashtag",
                    engagement_rate,
                    "rel_" + engagement_rate,
                    "group",
                    "hashtag_count",
                ]
            )
            most_eff_mentions_ig = pd.DataFrame(
                columns=PAGE_COLUMNS
                + [
                    "mention",
                    engagement_rate,
                    "rel_" + engagement_rate,
                    "group",
                    "mention_count",
                ]
            )
            compare_mentions_ig = pd.DataFrame(
                columns=PAGE_COLUMNS
                + [
                    "mention",
                    engagement_rate,
                    "rel_" + engagement_rate,
                    "group",
                    "mention_count",
                ]
            )
            return (
                most_eff_hashtags_ig,
                compare_hashtags_ig,
                most_eff_mentions_ig,
                compare_mentions_ig,
            )

    def construct_most_eff_words(
        self,
        df_most_eff_words_ig,
        lemmatize=False,
        engagement_rate="engagement_rate_by_post",
        grouped=True,
        min_count=2,
    ):
        """
        This function computes the output DataFrame for words, hashtags or mentions
        with their respective count and engagement rate.

        Parameters
        ----------
        df_most_eff_objects_ig:
            type: DataFrame
            DataFrame with words to extract.
        lemmatize:
            type: bool
            True if the lemmas are desired instead of words. default=False.
        engagement_rate:
            type: str
            Determines the column of engagement rate for the computations,
            default='engagement_rate_by_post'.
        grouped:
            type: bool
            Determines if the output is returned grouped by group or account,
            default=True.
        min_count:
            type: int
            Minimum number of counts on the word to consider in the analysis.

        Returns
        -------
        DataFrames
        """

        METHOD_NAME = "construct_most_eff_words"

        PAGE_COLUMNS = []
        if not grouped:
            PAGE_COLUMNS = ["owner_id", "owner_username"]

        try:
            words_ig = []
            lemmas_ig = []
            pos_tags_ig = []
            words_counts_ig = []
            words_eff_ig = []
            words_rel_eff_ig = []
            words_group_ig = []
            if PAGE_COLUMNS:
                words_ids_ig = []
                words_names_ig = []
            for _, row in df_most_eff_words_ig.iterrows():
                words_ig = words_ig + row.words
                lemmas_ig = lemmas_ig + row.lemmas
                pos_tags_ig = pos_tags_ig + row.pos_tags
                words_counts_ig = words_counts_ig + [1] * len(row.words)
                words_eff_ig = words_eff_ig + [row[engagement_rate]] * len(row.words)
                words_rel_eff_ig = words_rel_eff_ig + [
                    row["rel_" + engagement_rate]
                ] * len(row.words)
                words_group_ig = words_group_ig + [row.group] * len(row.words)
                if PAGE_COLUMNS:
                    words_ids_ig = words_ids_ig + [row[PAGE_COLUMNS[0]]] * len(
                        row.words
                    )
                    words_names_ig = words_names_ig + [row[PAGE_COLUMNS[1]]] * len(
                        row.words
                    )

            most_eff_words_ig = pd.DataFrame(
                {
                    "word": words_ig,
                    "lemma": lemmas_ig,
                    "pos_tag": pos_tags_ig,
                    "word_count": words_counts_ig,
                    engagement_rate: words_eff_ig,
                    "rel_" + engagement_rate: words_rel_eff_ig,
                    "group": words_group_ig,
                }
            )

            if PAGE_COLUMNS:
                most_eff_words_ig[PAGE_COLUMNS[0]] = words_ids_ig
                most_eff_words_ig[PAGE_COLUMNS[1]] = words_names_ig

            if lemmatize:
                word_column = "lemma"
            else:
                word_column = "word"
            most_eff_words_ig = (
                most_eff_words_ig[
                    PAGE_COLUMNS
                    + [
                        "pos_tag",
                        "group",
                        word_column,
                        "word_count",
                        engagement_rate,
                        "rel_" + engagement_rate,
                    ]
                ]
                .groupby(PAGE_COLUMNS + ["pos_tag", "group", word_column])
                .agg(
                    {
                        engagement_rate: "mean",
                        "rel_" + engagement_rate: "mean",
                        "word_count": "sum",
                    }
                )
            )
            most_eff_words_ig = most_eff_words_ig[
                most_eff_words_ig["word_count"] >= min_count
            ]

            most_eff_words_ig = most_eff_words_ig.sort_values(
                PAGE_COLUMNS + ["pos_tag", "group", engagement_rate], ascending=False
            ).reset_index()

            most_eff_nouns_ig = most_eff_words_ig[
                most_eff_words_ig["pos_tag"] == "NOUN"
            ]
            most_eff_verbs_ig = most_eff_words_ig[
                most_eff_words_ig["pos_tag"] == "VERB"
            ]
            most_eff_adjs_ig = most_eff_words_ig[most_eff_words_ig["pos_tag"] == "ADJ"]

            return (
                most_eff_words_ig,
                most_eff_nouns_ig,
                most_eff_verbs_ig,
                most_eff_adjs_ig,
            )

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            if lemmatize:
                word_column = "lemma"
            else:
                word_column = "word"
            most_eff_words_ig = pd.DataFrame(
                columns=PAGE_COLUMNS
                + [
                    "pos_tag",
                    "group",
                    word_column,
                    "word_count",
                    engagement_rate,
                    "rel_" + engagement_rate,
                ]
            )
            show_columns = PAGE_COLUMNS + [
                "group",
                "word",
                engagement_rate,
                "rel_" + engagement_rate,
                "word_count",
            ]
            most_eff_nouns_ig = pd.DataFrame(columns=show_columns)
            most_eff_verbs_ig = pd.DataFrame(columns=show_columns)
            most_eff_adjs_ig = pd.DataFrame(columns=show_columns)
            return (
                most_eff_words_ig,
                most_eff_nouns_ig,
                most_eff_verbs_ig,
                most_eff_adjs_ig,
            )

    def words(
        self,
        lemmatize=False,
        from_n_most_eff=None,
        min_count=2,
        engagement_rate="engagement_rate_by_post",
        grouped=True,
        **kwargs,
    ):
        """
        This function computes the following DataFrames:
          - 'most_eff_words_ig'
            which contains the words with higher associated
            engagement rate for the accounts selected, it is tagged with
            the part of speech tag of the word.

          - most_eff_nouns_ig, most_eff_verbs_ig and most_eff_adjs_ig
            which contains the nouns, verbs and adjectives with higher associated
            engagement rate for the accounts selected, it is tagged with
            the part of speech tag of the word.

        Parameters
        ----------
        lemmatize:
            type: bool
            True if the lemmas are desired instead of words. default=False.
        from_n_most_eff:
            type: int
            Number of posts to compute the ratios from, default=None means
            the computations is against all posts.
        engagement_rate:
            type: str
            Determines the column of engagement rate for the computations,
            default='engagement_rate_by_post'.
        **kwargs:
            account_ids:
                type: list
                Ids of the accounts to look for.
                If [] takes all the posts in the DataFrame posts.
            account_names:
                type: list
                Name of the accounts to look for.
                If [] takes all the posts in the DataFrame posts.

        Returns
        -------
        Tuple of DataFrames
        """

        METHOD_NAME = "words"

        if not self.features:
            self.features = Features()

        if not from_n_most_eff:
            from_n_most_eff = self.len_posts_full

        PAGE_COLUMNS = []
        if not grouped:
            PAGE_COLUMNS = ["owner_id", "owner_username"]

        if "account_ids" in kwargs.keys() and kwargs["account_ids"]:
            account_ids = kwargs["account_ids"]
            df_most_eff_words_ig = self.df_posts_full[
                self.df_posts_full.owner_id.isin(account_ids)
            ][
                PAGE_COLUMNS
                + ["caption", "group", engagement_rate, "rel_" + engagement_rate]
            ]
        elif "account_names" in kwargs.keys() and kwargs["account_names"]:
            account_names = kwargs["account_names"]
            df_most_eff_words_ig = self.df_posts_full[
                self.df_posts_full.owner_username.isin(account_names)
            ][
                PAGE_COLUMNS
                + ["caption", "group", engagement_rate, "rel_" + engagement_rate]
            ]
        else:
            df_most_eff_words_ig = deepcopy(
                self.df_posts_full[
                    PAGE_COLUMNS
                    + ["caption", "group", engagement_rate, "rel_" + engagement_rate]
                ]
            )

        try:
            df_most_eff_words_ig = df_most_eff_words_ig.sort_values(
                engagement_rate, ascending=False
            ).head(from_n_most_eff)

            if "processed_text" not in df_most_eff_words_ig.keys():
                df_most_eff_words_ig[
                    "processed_text"
                ] = df_most_eff_words_ig.caption.apply(
                    lambda msg: CleanText(msg).process_text(
                        mentions=True, hashtags=True, links=True, spec_chars=True
                    )
                )
            df_most_eff_words_ig["processed_text"] = df_most_eff_words_ig[
                "processed_text"
            ].apply(lambda txt: self.features.pos_tags(txt))

            df_most_eff_words_ig["words"] = df_most_eff_words_ig[
                "processed_text"
            ].apply(lambda pt: pt["words"])
            df_most_eff_words_ig["lemmas"] = df_most_eff_words_ig[
                "processed_text"
            ].apply(lambda pt: pt["lemmas"])
            df_most_eff_words_ig["pos_tags"] = df_most_eff_words_ig[
                "processed_text"
            ].apply(lambda pt: pt["pos_tags"])

            df_most_eff_words_ig = df_most_eff_words_ig[
                ~df_most_eff_words_ig["processed_text"].isna()
            ]

            (
                most_eff_words_ig,
                most_eff_nouns_ig,
                most_eff_verbs_ig,
                most_eff_adjs_ig,
            ) = self.construct_most_eff_words(
                df_most_eff_words_ig,
                lemmatize=lemmatize,
                engagement_rate=engagement_rate,
                grouped=grouped,
                min_count=min_count,
            )

            if PAGE_COLUMNS:
                most_eff_words_ig = most_eff_words_ig.rename(
                    columns={
                        PAGE_COLUMNS[0]: "_object_id",
                        PAGE_COLUMNS[1]: "_object_name",
                    }
                )
                most_eff_nouns_ig = most_eff_nouns_ig.rename(
                    columns={
                        PAGE_COLUMNS[0]: "_object_id",
                        PAGE_COLUMNS[1]: "_object_name",
                    }
                )
                most_eff_verbs_ig = most_eff_verbs_ig.rename(
                    columns={
                        PAGE_COLUMNS[0]: "_object_id",
                        PAGE_COLUMNS[1]: "_object_name",
                    }
                )
                most_eff_adjs_ig = most_eff_adjs_ig.reset_index().rename(
                    columns={
                        PAGE_COLUMNS[0]: "_object_id",
                        PAGE_COLUMNS[1]: "_object_name",
                    }
                )
                PAGE_COLUMNS = ["_object_id", "_object_name"]

            show_columns = PAGE_COLUMNS + [
                "group",
                "word",
                engagement_rate,
                "rel_" + engagement_rate,
                "word_count",
            ]
            return (
                most_eff_words_ig,
                most_eff_nouns_ig[show_columns],
                most_eff_verbs_ig[show_columns],
                most_eff_adjs_ig[show_columns],
            )

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            if PAGE_COLUMNS:
                PAGE_COLUMNS = ["_object_id", "_object_name"]
            most_eff_words_ig = pd.DataFrame(
                columns=PAGE_COLUMNS
                + [
                    "pos_tag",
                    "group",
                    "word",
                    "word_count",
                    engagement_rate,
                    "rel_" + engagement_rate,
                ]
            )
            show_columns = PAGE_COLUMNS + [
                "group",
                "word",
                engagement_rate,
                "rel_" + engagement_rate,
                "word_count",
            ]
            most_eff_nouns_ig = pd.DataFrame(columns=show_columns)
            most_eff_verbs_ig = pd.DataFrame(columns=show_columns)
            most_eff_adjs_ig = pd.DataFrame(columns=show_columns)
            return (
                most_eff_words_ig,
                most_eff_nouns_ig,
                most_eff_verbs_ig,
                most_eff_adjs_ig,
            )
